
/*******************************************************************************
*
* File check3.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the programs ipt_blk() and dist_bndry()
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "start.h"
#include "block.h"
#include "global.h"


int main(int argc,char *argv[])
{
   int my_rank;
   int bo[4],bs[4];
   int ix,mu,d1,d2,itest;
   int x0,x1,x2,x3,x[4];
   block_t b;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check3.log","w",stdout);
      in=freopen("check3.in","r",stdin);

      printf("\n");
      printf("Check of the programs ipt_blk() and dist_bndry()\n");
      printf("------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bo %d %d %d %d\n",&bo[0],&bo[1],&bo[2],&bo[3]);
      scanf("bs %d %d %d %d",&bs[0],&bs[1],&bs[2],&bs[3]);
      fclose(in);

      printf("bo = %d %d %d %d\n",bo[0],bo[1],bo[2],bo[3]);
      printf("bs = %d %d %d %d\n",bs[0],bs[1],bs[2],bs[3]);
   }

   MPI_Bcast(bo,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);

   geometry();
   alloc_blk(&b,bo,bs,0,0,0,0);

   itest=0;

   for (x0=0;x0<bs[0];x0++)
   {
      for (x1=0;x1<bs[1];x1++)
      {
         for (x2=0;x2<bs[2];x2++)
         {
            for (x3=0;x3<bs[3];x3++)
            {
               x[0]=x0;
               x[1]=x1;
               x[2]=x2;
               x[3]=x3;

               ix=ipt_blk(&b,x);
               d1=dist_bndry(&b,ix);

               d2=bs[0]+bs[1]+bs[2]+bs[3];

               for (mu=0;mu<4;mu++)
               {
                  if (d2>(x[mu]+1))
                     d2=x[mu]+1;
                  if (d2>(bs[mu]-x[mu]))
                     d2=bs[mu]-x[mu];
               }

               if (d1!=d2)
                  itest=1;
            }
         }
      }
   }

   fflush(log);
   d1=dist_bndry(&b,b.vol);

   error(itest==1,1,"main [check3.c]","Incorrect distance (block points)");
   error(d1!=0,1,"main [check3.c]","Incorrect distance (boundary points)");
   error_chk();

   if (my_rank==0)
   {
      printf("\n");
      printf("No errors detected\n\n");
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
