
/*******************************************************************************
*
* File check5.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the assignment programs assign_u2ubgr, ...
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "flags.h"
#include "start.h"
#include "random.h"
#include "block.h"
#include "global.h"


static int ipts[VOLUME];
static su3 ubnd[BNDRY+NPROC%2];
static su3_dble udbnd[BNDRY+NPROC%2];
static su3_dble udsve[4*VOLUME];


static void cp_ubnd(void)
{
   int ifc,n,mu,nfc[8];
   int ixb,ix,iy,iz;
   int saddr,raddr,np,nbf,tag;
   float *sbuf,*rbuf;
   su3 *ub;
   MPI_Status stat;

   nfc[0]=FACE0;
   nfc[1]=FACE0;
   nfc[2]=FACE1;
   nfc[3]=FACE1;
   nfc[4]=FACE2;
   nfc[5]=FACE2;
   nfc[6]=FACE3;
   nfc[7]=FACE3;

   ixb=BNDRY/2;
   ub=ubnd;

   for (ifc=0;ifc<8;ifc++)
   {
      n=nfc[ifc]/2;

      for (ix=ixb;ix<(ixb+n);ix++)
      {
         iy=map[ix];
         *ub=*(pu[iy][0]+ifc);
         ub+=1;
      }

      ixb+=n;
   }

   ub=ubnd;

   for (ifc=0;ifc<8;ifc++)
   {
      n=nfc[ifc]/2;

      if (n>0)
      {
         mu=ifc/2;

         if (ifc==(2*mu))
            saddr=npr[ifc+1];
         else
            saddr=npr[ifc-1];
         raddr=npr[ifc];

         np=cpr[mu];
         nbf=18*n;
         tag=mpi_tag();
         sbuf=(float*)(ub);
         rbuf=(float*)(ub+(BNDRY/2));

         if ((np|0x1)!=np)
         {
            MPI_Send(sbuf,nbf,MPI_FLOAT,saddr,tag,MPI_COMM_WORLD);
            MPI_Recv(rbuf,nbf,MPI_FLOAT,raddr,tag,MPI_COMM_WORLD,&stat);
         }
         else
         {
            MPI_Recv(rbuf,nbf,MPI_FLOAT,raddr,tag,MPI_COMM_WORLD,&stat);
            MPI_Send(sbuf,nbf,MPI_FLOAT,saddr,tag,MPI_COMM_WORLD);
         }

         ub+=n;
      }
   }

   ixb=0;
   ub=ubnd;

   for (ifc=0;ifc<8;ifc++)
   {
      n=nfc[ifc]/2;
      mu=ifc/2;

      for (ix=ixb;ix<(ixb+n);ix++)
      {
         iy=map[ix];

         if (ifc==(2*mu))
         {
            iz=map[iup[iy][mu]-VOLUME];
            *ub=*(pu[iz][0]+ifc+1);
         }
         else
         {
            iz=map[idn[iy][mu]-VOLUME];
            *ub=*(pu[iz][0]+ifc-1);
         }

         ub+=1;
      }

      ixb+=n;
   }
}


static void cp_udbnd(void)
{
   int ifc,n,mu,nfc[8];
   int ixb,ix,iy,iz;
   int saddr,raddr,np,nbf,tag;
   double *sbuf,*rbuf;
   su3_dble *ub;
   MPI_Status stat;

   nfc[0]=FACE0;
   nfc[1]=FACE0;
   nfc[2]=FACE1;
   nfc[3]=FACE1;
   nfc[4]=FACE2;
   nfc[5]=FACE2;
   nfc[6]=FACE3;
   nfc[7]=FACE3;

   ixb=BNDRY/2;
   ub=udbnd;

   for (ifc=0;ifc<8;ifc++)
   {
      n=nfc[ifc]/2;

      for (ix=ixb;ix<(ixb+n);ix++)
      {
         iy=map[ix];
         *ub=*(pud[iy][0]+ifc);
         ub+=1;
      }

      ixb+=n;
   }

   ub=udbnd;

   for (ifc=0;ifc<8;ifc++)
   {
      n=nfc[ifc]/2;

      if (n>0)
      {
         mu=ifc/2;

         if (ifc==(2*mu))
            saddr=npr[ifc+1];
         else
            saddr=npr[ifc-1];
         raddr=npr[ifc];

         np=cpr[mu];
         nbf=18*n;
         tag=mpi_tag();
         sbuf=(double*)(ub);
         rbuf=(double*)(ub+(BNDRY/2));

         if ((np|0x1)!=np)
         {
            MPI_Send(sbuf,nbf,MPI_DOUBLE,saddr,tag,MPI_COMM_WORLD);
            MPI_Recv(rbuf,nbf,MPI_DOUBLE,raddr,tag,MPI_COMM_WORLD,&stat);
         }
         else
         {
            MPI_Recv(rbuf,nbf,MPI_DOUBLE,raddr,tag,MPI_COMM_WORLD,&stat);
            MPI_Send(sbuf,nbf,MPI_DOUBLE,saddr,tag,MPI_COMM_WORLD);
         }

         ub+=n;
      }
   }

   ixb=0;
   ub=udbnd;

   for (ifc=0;ifc<8;ifc++)
   {
      n=nfc[ifc]/2;
      mu=ifc/2;

      for (ix=ixb;ix<(ixb+n);ix++)
      {
         iy=map[ix];

         if (ifc==(2*mu))
         {
            iz=map[iup[iy][mu]-VOLUME];
            *ub=*(pud[iz][0]+ifc+1);
         }
         else
         {
            iz=map[idn[iy][mu]-VOLUME];
            *ub=*(pud[iz][0]+ifc-1);
         }

         ub+=1;
      }

      ixb+=n;
   }
}


static int cmp_u(su3 *u,su3 *v)
{
   int n;
   float *ru,*rv;

   ru=(float*)(u);
   rv=(float*)(v);

   for (n=0;n<18;n++)
   {
      if ((*ru)!=(*rv))
         return 1;

      ru+=1;
      rv+=1;
   }

   return 0;
}


static int cmp_ud(su3_dble *u,su3_dble *v)
{
   int n;
   double *ru,*rv;

   ru=(double*)(u);
   rv=(double*)(v);

   for (n=0;n<18;n++)
   {
      if ((*ru)!=(*rv))
         return 1;

      ru+=1;
      rv+=1;
   }

   return 0;
}


static int cmp_u2one(su3 *u)
{
   int n;
   float *ru;

   ru=(float*)(u);

   for (n=0;n<18;n++)
   {
      if (((n%8)==0)&&((*ru)!=1.0f))
         return 1;
      if (((n%8)!=0)&&((*ru)!=0.0f))
         return 1;

      ru+=1;
   }

   return 0;
}


static int cmp_ud2one(su3_dble *u)
{
   int n;
   double *ru;

   ru=(double*)(u);

   for (n=0;n<18;n++)
   {
      if (((n%8)==0)&&((*ru)!=1.0))
         return 1;
      if (((n%8)!=0)&&((*ru)!=0.0))
         return 1;

      ru+=1;
   }

   return 0;
}


static int check_ub(block_t *b)
{
   int ix,iy,iz,mu,ifc;
   su3 *u;
   bndry_t *bb;

   u=(*b).u[0];

   for (ix=(*b).volh;ix<(*b).vol;ix++)
   {
      iy=(*b).imb[ix];

      for (mu=0;mu<4;mu++)
      {
         if ((*b).iup[ix][mu]!=(*b).vol)
         {
            if (cmp_u(u,pu[iy][mu])==1)
               return 1;
         }
         else
         {
            if (cmp_u2one(u)==1)
               return 1;
         }

         u+=1;
         iz=idn[iy][mu];

         if ((*b).idn[ix][mu]!=(*b).vol)
         {
            if (cmp_u(u,pu[iz][mu])==1)
               return 1;
         }
         else
         {
            if (cmp_u2one(u)==1)
               return 1;
         }

         u+=1;
      }
   }

   bb=(*b).bb;

   for (ifc=0;ifc<8;ifc++)
   {
      mu=ifc/2;
      u=(*bb).u;

      if ((*bb).ibn==0)
      {
         for (ix=0;ix<(*bb).vol;ix++)
         {
            iy=(*bb).imb[ix];
            iz=idn[iy][mu];

            if (ifc==(2*mu))
            {
               if (cmp_u(u,pu[iy][mu])==1)
                  return 1;
            }
            else
            {
               if (cmp_u(u,pu[iz][mu])==1)
                  return 1;
            }

            u+=1;
         }
      }
      else
      {
         for (ix=0;ix<(*bb).vol;ix++)
         {
            iy=(*bb).imb[ix];

            if (ifc==(2*mu))
            {
               iz=map[iup[iy][mu]-VOLUME];
               iz=idn[iz][mu];
            }
            else
            {
               iz=map[idn[iy][mu]-VOLUME];
               iz=iup[iz][mu];
            }

            if (cmp_u(u,ubnd+iz-VOLUME)==1)
               return 2;
            u+=1;
         }
      }

      bb+=1;
   }

   return 0;
}


static int check_udb(block_t *b)
{
   int ix,iy,iz,mu,ifc;
   su3_dble *u;
   bndry_t *bb;

   u=(*b).ud[0];

   for (ix=(*b).volh;ix<(*b).vol;ix++)
   {
      iy=(*b).imb[ix];

      for (mu=0;mu<4;mu++)
      {
         if ((*b).iup[ix][mu]!=(*b).vol)
         {
            if (cmp_ud(u,pud[iy][mu])==1)
               return 1;
         }
         else
         {
            if (cmp_ud2one(u)==1)
               return 1;
         }

         u+=1;
         iz=idn[iy][mu];

         if ((*b).idn[ix][mu]!=(*b).vol)
         {
            if (cmp_ud(u,pud[iz][mu])==1)
               return 1;
         }
         else
         {
            if (cmp_ud2one(u)==1)
               return 1;
         }

         u+=1;
      }
   }

   bb=(*b).bb;

   for (ifc=0;ifc<8;ifc++)
   {
      mu=ifc/2;
      u=(*bb).ud;

      if ((*bb).ibn==0)
      {
         for (ix=0;ix<(*bb).vol;ix++)
         {
            iy=(*bb).imb[ix];
            iz=idn[iy][mu];

            if (ifc==(2*mu))
            {
               if (cmp_ud(u,pud[iy][mu])==1)
                  return 1;
            }
            else
            {
               if (cmp_ud(u,pud[iz][mu])==1)
                  return 1;
            }

            u+=1;
         }
      }
      else
      {
         for (ix=0;ix<(*bb).vol;ix++)
         {
            iy=(*bb).imb[ix];

            if (ifc==(2*mu))
            {
               iz=map[iup[iy][mu]-VOLUME];
               iz=idn[iz][mu];
            }
            else
            {
               iz=map[idn[iy][mu]-VOLUME];
               iz=iup[iz][mu];
            }

            if (cmp_ud(u,udbnd+iz-VOLUME)==1)
               return 2;
            u+=1;
         }
      }

      bb+=1;
   }

   return 0;
}


static int check_ubgr(blk_grid_t grid)
{
   int nb,isw,ifail;
   block_t *b,*bm;

   b=blk_list(grid,&nb,&isw);
   bm=b+nb;
   ifail=0;

   for (;b<bm;b++)
      ifail|=check_ub(b);

   return ifail;
}


static int check_udbgr(blk_grid_t grid)
{
   int nb,isw,ifail;
   block_t *b,*bm;

   b=blk_list(grid,&nb,&isw);
   bm=b+nb;
   ifail=0;

   for (;b<bm;b++)
      ifail|=check_udb(b);

   return ifail;
}


static void save_ud(void)
{
   su3_dble *ud,*uds,*um;

   ud=pud[VOLUME/2][0];
   uds=udsve;
   um=ud+4*VOLUME;

   for (;ud<um;ud++)
   {
      *uds=*ud;
      uds+=1;
   }
}


static void restore_ud(void)
{
   su3_dble *ud,*uds,*um;

   ud=pud[VOLUME/2][0];
   uds=udsve;
   um=ud+4*VOLUME;

   for (;ud<um;ud++)
   {
      *ud=*uds;
      uds+=1;
   }
}


static int cmp_udsve(void)
{
   su3_dble *ud,*uds,*um;

   ud=pud[VOLUME/2][0];
   uds=udsve;
   um=ud+4*VOLUME;

   for (;ud<um;ud++)
   {
      if (cmp_ud(ud,uds)!=0)
         return 1;

      uds+=1;
   }

   return 0;
}


static void set_ipts(int *bs)
{
   int x0,x1,x2,x3,ix;
   int y0,y1,y2,y3;

   for (x0=0;x0<L0;x0++)
   {
      for (x1=0;x1<L1;x1++)
      {
         for (x2=0;x2<L2;x2++)
         {
            for (x3=0;x3<L3;x3++)
            {
               ix=x0;
               ix=L1*ix+x1;
               ix=L2*ix+x2;
               ix=L3*ix+x3;
               ix=ipt[ix];

               y0=x0%bs[0];
               y1=x1%bs[1];
               y2=x2%bs[2];
               y3=x3%bs[3];

               y0=((y0>=1)&&(y0<=(bs[0]-2)));
               y1=((y1>=1)&&(y1<=(bs[1]-2)));
               y2=((y2>=1)&&(y2<=(bs[2]-2)));
               y3=((y3>=1)&&(y3<=(bs[3]-2)));

               if ((y0==0)||(y1==0)||(y2==0)||(y3==0))
                  ipts[ix]=0;
               else
                  ipts[ix]=1;
            }
         }
      }
   }
}


static int cmp_udsve_inactive(int *bs)
{
   int ix,iy,mu;
   su3_dble *ud,*uds;

   set_ipts(bs);

   ud=pud[VOLUME/2][0];
   uds=udsve;

   for (ix=(VOLUME/2);ix<VOLUME;ix++)
   {
      if (ipts[ix]==1)
      {
         ud+=8;
         uds+=8;
      }
      else
      {
         for (mu=0;mu<4;mu++)
         {
            iy=iup[ix][mu];
            if ((iy>=VOLUME)||(ipts[iy]==0))
            {
               if (cmp_ud(ud,uds)!=0)
                  return 1;
            }

            ud+=1;
            uds+=1;

            iy=idn[ix][mu];
            if ((iy>=VOLUME)||(ipts[iy]==0))
            {
               if (cmp_ud(ud,uds)!=0)
                  return 1;
            }

            ud+=1;
            uds+=1;
         }
      }
   }

   return 0;
}


static void random_active_blk(block_t *b)
{
   int ix,iy,mu,dx,dy;
   su3_dble *ud;

   for (ix=(*b).volh;ix<(*b).vol;ix++)
   {
      ud=(*b).ud[ix-(*b).volh];
      dx=dist_bndry(b,ix);

      for (mu=0;mu<4;mu++)
      {
         iy=((*b).iup)[ix][mu];
         dy=dist_bndry(b,iy);

         if ((dx>=2)||(dy>=2))
            random_su3_dble(ud);

         ud+=1;

         iy=((*b).idn)[ix][mu];
         dy=dist_bndry(b,iy);

         if ((dx>=2)||(dy>=2))
            random_su3_dble(ud);

         ud+=1;
      }
   }
}


static void random_active_bgr(blk_grid_t grid)
{
   int nb,isw;
   block_t *b,*bm;

   b=blk_list(grid,&nb,&isw);
   bm=b+nb;

   for (;b<bm;b++)
      random_active_blk(b);
}


int main(int argc,char *argv[])
{
   int my_rank,n,nb,isw;
   int bs[4];
   block_t *b;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check5.log","w",stdout);
      in=freopen("check5.in","r",stdin);

      printf("\n");
      printf("Check of the assignment programs assign_u2ubgr, ...\n");
      printf("---------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bs %d %d %d %d",&bs[0],&bs[1],&bs[2],&bs[3]);
      fclose(in);

      printf("bs = %d %d %d %d\n",bs[0],bs[1],bs[2],bs[3]);
   }

   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);

   start_ranlux(0,1234);
   geometry();
   alloc_u();
   alloc_ud();

   alloc_bgr(GCR_BLOCKS,bs);
   alloc_bgr(HMC_BLOCKS,bs);

   random_u();
   random_ud();
   cp_ubnd();
   cp_udbnd();

   assign_u2ubgr(GCR_BLOCKS);
   error(check_ubgr(GCR_BLOCKS)!=0,1,"main [check5.c]",
         "assign_u2ubgr is incorrect");

   assign_ud2udbgr(HMC_BLOCKS);
   error(check_udbgr(HMC_BLOCKS)!=0,1,"main [check5.c]",
         "assign_ud2udbgr is incorrect");

   assign_ud2ubgr(GCR_BLOCKS);
   assign_ud2u();
   cp_ubnd();
   error(check_ubgr(GCR_BLOCKS)!=0,1,"main [check5.c]",
         "assign_ud2ubgr is incorrect");

   b=blk_list(HMC_BLOCKS,&nb,&isw);
   random_ud();
   assign_ud2u();
   cp_ubnd();
   assign_ud2udbgr(HMC_BLOCKS);

   for (n=0;n<nb;n++)
   {
      assign_udb2ub(HMC_BLOCKS,n);
      error(check_ub(b+n)!=0,1,"main [check5.c]",
            "assign_udb2ub on block number %d is incorrect",n);
   }

   save_ud();
   random_active_bgr(HMC_BLOCKS);
   swap_active(HMC_BLOCKS);
   error(cmp_udsve_inactive(bs)!=0,1,"main [check5.c]",
         "swap_active is incorrect (overwrite inactive link variables)");

   swap_active(HMC_BLOCKS);
   error(cmp_udsve()!=0,1,"main [check5.c]",
         "swap_active is incorrect (swap*swap does not restore)");

   swap_active(HMC_BLOCKS);
   cp_udbnd();
   save_ud();
   swap_active(HMC_BLOCKS);
   restore_ud();
   error(check_udbgr(HMC_BLOCKS)!=0,1,"main [check5.c]",
         "swap_active is incorrect (mapping to the block fields)");

   error_chk();

   if (my_rank==0)
   {
      printf("\n");
      printf("No errors detected --- all programs work correctly\n\n");
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
