
/*******************************************************************************
*
* File check7b.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Comparision of Qoe_blk, ... with Qoe, ...
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "flags.h"
#include "random.h"
#include "start.h"
#include "linalg.h"
#include "block.h"
#include "sw_term.h"
#include "dirac.h"
#include "global.h"


int main(int argc,char *argv[])
{
   int my_rank,n0,n,nb,isw,bs[4];
   float d,doe,deo,dno,dht;
   complex z;
   spinor *ps0e,*ps1e,*ps2e;
   spinor *ps0o,*ps1o,*ps2o;
   block_t *b0,*b;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check7b.log","w",stdout);
      in=freopen("check7b.in","r",stdin);

      printf("\n");
      printf("Comparision of Qoe_blk, ... with Qoe, ...\n");
      printf("-----------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bs %d %d %d %d",&bs[0],&bs[1],&bs[2],&bs[3]);
      fclose(in);

      printf("bs = %d %d %d %d\n\n",bs[0],bs[1],bs[2],bs[3]);
   }

   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);

   start_ranlux(0,1234);
   geometry();
   alloc_u();
   alloc_ud();
   alloc_s(2);
   alloc_sw();
   alloc_swd();

   random_ud();
   set_lat_parms(0.0,0.1,0.3);
   sw_term();
   assign_ud2u();
   assign_swd2sw();

   alloc_bgr(GCR_BLOCKS,bs);
   assign_u2ubgr(GCR_BLOCKS);
   assign_swd2swbgr(GCR_BLOCKS,NO_PTS);

   z.re=-1.0f;
   z.im=0.0f;
   doe=0.0f;
   deo=0.0f;
   dno=0.0f;
   dht=0.0f;

   b0=blk_list(GCR_BLOCKS,&nb,&isw);

   for (n0=0;n0<nb;n0++)
   {
      if (isw==0)
         n=n0;
      else
         n=(n0+(nb/2))%nb;
      b=b0+n;

      ps0e=&(*b).s[0][0];
      ps1e=&(*b).s[1][0];
      ps2e=&(*b).s[2][0];

      ps0o=&(*b).s[0][(*b).volh];
      ps1o=&(*b).s[1][(*b).volh];
      ps2o=&(*b).s[2][(*b).volh];

      set_s2zero(VOLUME,ps[0][0]);
      random_s((*b).vol,ps0e,1.0f);
      assign_sb2s(b,EVEN_PTS,0,0);

      error(check_blk_flags(GCR_BLOCKS,QOE,n)!=0,1,"main [check7b.c]",
            "Fields are not in proper condition [Qoe_blk]");
      Qoe_blk(b,0,1);
      error(check_flags(QOE)!=0,1,"main [check7b.c]",
            "Fields are not in proper condition [Qoe]");
      Qoe(0,1);
      assign_s2sb(b,ODD_PTS,1,2);

      mulc_spinor_add((*b).volh,ps2o,ps1o,z);
      d=norm_square((*b).volh,1,ps2o)/norm_square((*b).volh,1,ps0o);
      if (d>doe)
         doe=d;

      random_s((*b).vol,ps0e,1.0f);
      random_s((*b).vol,ps1e,1.0f);
      assign_sb2s(b,ODD_PTS,0,0);
      assign_sb2s(b,EVEN_PTS,1,1);

      error(check_blk_flags(GCR_BLOCKS,QEO,n)!=0,1,"main [check7b.c]",
            "Fields are not in proper condition [Qeo_blk]");
      Qeo_blk(b,0,1);
      error(check_flags(QEO)!=0,1,"main [check7b.c]",
            "Fields are not in proper condition [Qeo]");
      Qeo(0,1);
      assign_s2sb(b,EVEN_PTS,1,2);

      mulc_spinor_add((*b).volh,ps2e,ps1e,z);
      d=norm_square((*b).volh,1,ps2e)/norm_square((*b).volh,1,ps0e);
      if (d>deo)
         deo=d;

      random_s((*b).vol,ps0e,1.0f);
      assign_sb2s(b,ALL_PTS,0,0);

      error(check_blk_flags(GCR_BLOCKS,QNOHAT,n)!=0,1,"main [check7b.c]",
            "Fields are not in proper condition [Qnohat_blk]");
      Qnohat_blk(b,0,1);
      error(check_flags(QNOHAT)!=0,1,"main [check7b.c]",
            "Fields are not in proper condition [Qnohat]");
      Qnohat(0,1);
      assign_s2sb(b,ALL_PTS,1,2);

      mulc_spinor_add((*b).vol,ps2e,ps1e,z);
      d=norm_square((*b).vol,1,ps2e)/norm_square((*b).vol,1,ps0e);
      if (d>dno)
         dno=d;
   }

   error(assign_swd2swbgr(GCR_BLOCKS,ODD_PTS)!=0,1,"main [check7b.c]",
         "Inversion of swd on the odd sites was not safe");

   for (n0=0;n0<nb;n0++)
   {
      if (isw==0)
         n=n0;
      else
         n=(n0+(nb/2))%nb;
      b=b0+n;

      ps0e=&(*b).s[0][0];
      ps1e=&(*b).s[1][0];
      ps2e=&(*b).s[2][0];
      ps1o=&(*b).s[1][(*b).volh];

      random_s((*b).vol,ps0e,1.0f);
      assign_s2s((*b).volh,ps0e,ps2e);
      apply_sw((*b).volh,(*b).sw,ps2e,ps2e);
      Qoe_blk(b,0,1);
      apply_sw((*b).volh,(*b).sw+(*b).vol,ps1o,ps1o);
      Qeo_blk(b,1,2);
      error(check_blk_flags(GCR_BLOCKS,QHAT,n)!=0,1,"main [check7b.c]",
            "Fields are not in proper condition [Qhat_blk]");
      Qhat_blk(b,0,1);

      mulc_spinor_add((*b).volh,ps1e,ps2e,z);
      d=norm_square((*b).volh,1,ps1e)/norm_square((*b).volh,1,ps0e);
      if (d>dht)
         dht=d;
   }

   error_chk();

   if (my_rank==0)
   {
      printf("|[Qoe_blk-Qoe]*psi|/|psi| = %.2e\n\n",
             sqrt((double)(doe)));

      printf("|[Qeo_blk-Qeo]*psi|/|psi| = %.2e\n\n",
             sqrt((double)(deo)));

      printf("|[Qnohat_blk-Qnohat]*psi|/|psi| = %.2e\n\n",
             sqrt((double)(dno)));

      printf("|[Qhat_blk-(Qee_blk-Qeo_blk*Qooinv_blk*Qoe_blk)]*psi|/|psi| ");
      printf("= %.2e\n\n",sqrt((double)(dht)));

      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
