
/*******************************************************************************
*
* File time1n.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of Qnohat
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "flags.h"
#include "random.h"
#include "start.h"
#include "sw_term.h"
#include "dirac.h"
#include "global.h"


int main(int argc,char *argv[])
{
   int my_rank,count,n;
   double wt1,wt2,wdt;
   FILE *log=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("time1n.log","w",stdout);

      printf("\n");
      printf("Timing of Qnohat\n");
      printf("----------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);
   }

   start_ranlux(0,12345);
   geometry();
   alloc_u();
   alloc_ud();
   alloc_s(2);
   alloc_sw();
   alloc_swd();

   random_ud();
   assign_ud2u();
   set_lat_parms(0.0,0.1,0.2);
   sw_term();
   assign_swd2sw();

   random_s(VOLUME,ps[0][0],1.0f);
   random_s(VOLUME,ps[1][0],1.0f);

   n=1000000/VOLUME;
   if (n<1)
      n=1;

   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();
   for (count=0;count<n;count++)
      Qnohat(0,1);
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   error_chk();
   wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);

   if (my_rank==0)
   {
      printf("Time per lattice point: %4.3f micro sec",wdt);
      printf(" (%d Mflops [%d bit arithmetic])\n\n",
             (int)(1920.0/wdt),(int)sizeof(spinor)/3);
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
