
/*******************************************************************************
*
* File check1.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of set_alg2zero, norm_square_alg and random_alg
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "hmc.h"
#include "global.h"
   
#define NMOM 100000

static double var[64],var_all[64];


int main(int argc,char *argv[])
{
   int my_rank,n,i,j;
   double d,dmax,dmax_all,*r,sm;
   su3_vector_dble *v;
   su3_dble *M,*m,w;
   su3_alg_dble *X,*x;
   FILE *log=NULL;   

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check1.log","w",stdout); 
      printf("\n");
      printf("Check of set_alg2zero, norm_square_alg and random_alg\n");
      printf("-----------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);
      
      printf("Number of momenta: %d\n\n",NMOM);
   }

   start_ranlux(0,123456);
   geometry();

   X=amalloc(NMOM*sizeof(su3_alg_dble),4);
   M=amalloc(NMOM*sizeof(su3_dble),4);   
   error((X==NULL)||(M==NULL),1,
         "main [check1.c]","Unable to allocate field arrays");

   set_alg2zero(NMOM,X);
   dmax=0.0;

   for (n=0;n<NMOM;n++)
   {
      r=(double*)(X+n);

      for (i=0;i<8;i++)
      {
         d=fabs(r[i]);
         if (d>dmax)
            dmax=d;
      }

      X[n].c3=1.0;
   }
   
   MPI_Reduce(&dmax,&dmax_all,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
   
   if (my_rank==0)
   {
      printf("Allocate momentum fields\n");
      printf("max |X| = %.1e\n\n",dmax_all);
   }

   dmax=fabs(norm_square_alg(NMOM,X)-4.0*(double)(NMOM));
   MPI_Reduce(&dmax,&dmax_all,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);

   if (my_rank==0)
   {
      printf("Check of norm_square_alg:\n");
      printf("Element count = %.1e (should be 0.0)\n\n",dmax_all);
      printf("Choose random elements\n");
   }   

   sm=0.0;
   dmax=0.0;
   
   for (n=0;n<NMOM;n++)
   {
      x=X+n;
      m=M+n;
      
      random_alg(1,x);

      (*m).c11.re=0.0;
      (*m).c11.im=(*x).c1+(*x).c2;
      (*m).c22.re=0.0;
      (*m).c22.im=(*x).c2-2.0*(*x).c1;
      (*m).c33.re=0.0;
      (*m).c33.im=(*x).c1-2.0*(*x).c2;

      (*m).c12.re= (*x).c3;
      (*m).c12.im= (*x).c4;
      (*m).c21.re=-(*x).c3;
      (*m).c21.im= (*x).c4;

      (*m).c13.re= (*x).c5;
      (*m).c13.im= (*x).c6;
      (*m).c31.re=-(*x).c5;
      (*m).c31.im= (*x).c6;
      
      (*m).c23.re= (*x).c7;
      (*m).c23.im= (*x).c8;
      (*m).c32.re=-(*x).c7;
      (*m).c32.im= (*x).c8;
      
      v=(su3_vector_dble*)(m);

      d=norm_square_alg(1,x);
      sm+=d;

      _su3_times_su3(w,(*m),(*m));
      d=fabs(1.0+2.0*(w.c11.re+w.c22.re+w.c33.re)/d);
      if (d>dmax)
         dmax=d;
   }

   MPI_Reduce(&dmax,&dmax_all,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);

   if (my_rank==0)
   {
      printf("|1.0-2*tr{X^2}/||X||^2| = %.1e (single elements)\n",dmax_all);
      printf("(should be less than 1.0e-15 or so)\n");
   }
   
   dmax=fabs(1.0-sm/norm_square_alg(NMOM,X));
   MPI_Reduce(&dmax,&dmax_all,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
   
   if (my_rank==0)
   {
      printf("|1.0-2*tr{X^2}/||X||^2| = %.1e (whole vector)\n",dmax_all);
      printf("(should be less than %1.1e or so)\n\n",
             1.0e-15*sqrt((double)(NMOM)));
      
      printf("Random momentum field with Gaussian distribution:\n\n");
   }
   
   random_alg(NMOM,X);

   for (i=0;i<8;i++)
   {
      for (j=0;j<8;j++)
         var[8*i+j]=0.0;
   }

   for (n=0;n<NMOM;n++)
   {
      r=(double*)(X+n);

      for (i=0;i<8;i++)
      {
         for (j=i;j<8;j++)
            var[8*i+j]+=r[i]*r[j];
      }
   }

   MPI_Reduce(var,var_all,64,MPI_DOUBLE,MPI_SUM,0,MPI_COMM_WORLD);
      
   if (my_rank==0)
   {
      dmax=0.0;
      
      for (i=0;i<8;i++)
      {
         for (j=i;j<8;j++)
         {
            if ((i==j)&&(i>1))
               d=1.0/4.0;
            else if (i==j)
               d=1.0/9.0;
            else if ((i==0)&&(j==1))
               d=1.0/18.0;
            else
               d=0.0;
            
            var_all[8*i+j]/=(double)(NMOM*NPROC);

            if (d!=0.0)
               printf("<b[%d]*b[%d]> = % .4e (expect %.4e)\n",
                      i,j,var_all[8*i+j],d);
            else
            {
               d=fabs(var_all[8*i+j]);

               if (d>dmax)
                  dmax=d;
            }
         }
      }

      printf("\n");
      printf("max|<b[i]*b[j]>| = %.1e (should be 0.0 within errors)\n\n",dmax);
      fclose(log);
   }

   MPI_Finalize();   
   exit(0);
}
