
/*******************************************************************************
*
* File check10.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of hmc_gcr_vec(), hmc_gcr_frf() and hmc_gcr_acf()
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "block.h"
#include "linalg.h"
#include "dirac.h"
#include "sw_term.h"
#include "hmc.h"
#include "global.h"

static const su3_dble ud0={{0.0}};


static void init_ud(void)
{
   su3_alg_dble X;
   su3_dble *u,*um;

   u=pud[VOLUME/2][0];
   um=u+4*VOLUME;

   for (;u<um;u++)
   {
      *u=ud0;
      (*u).c11.re=1.0;
      (*u).c22.re=1.0;
      (*u).c33.re=1.0;      
      random_alg(1,&X);
      ExpX(0.1,&X,u);      
   }

   set_flags(NEW_UD);
   renormalize_ud();
}


static void init_mom(int n)
{
   hmc_t *hflds;

   hflds=hmcflds()+n;
   random_alg((*hflds).nlks,(*hflds).mom);
}


static void rot_ud(int n,double t)
{
   int nb,isw,*lk,*lkm;
   su3_dble *ub;
   su3_alg_dble *mom;
   block_t *b;
   hmc_t *hflds;

   b=blk_list(HMC_BLOCKS,&nb,&isw)+n;
   hflds=hmcflds()+n;
   
   ub=(*b).ud[0];
   mom=(*hflds).mom;
   lk=(*hflds).lks;
   lkm=lk+(*hflds).nlks;
   
   for (;lk<lkm;lk++)
   {
      ExpX(t,mom,ub+(*lk));
      mom+=1;
   }

   set_blk_flags(HMC_BLOCKS,NEW_ACTIVE_UD,n);
}


static double dSdt(int n)
{
   double sm;
   su3_alg_dble *fr,*frm,*mom;
   hmc_t *hflds;

   hflds=hmcflds()+n;

   fr=(*hflds).frc;
   frm=fr+(*hflds).nlks;
   mom=(*hflds).mom;   
   sm=0.0;
   
   for (;fr<frm;fr++)
   {
      sm+=12.0*((*mom).c1*(*fr).c1+(*mom).c2*(*fr).c2);
      sm-= 6.0*((*mom).c1*(*fr).c2+(*mom).c2*(*fr).c1);
      sm+= 4.0*((*mom).c3*(*fr).c3+(*mom).c4*(*fr).c4+(*mom).c5*(*fr).c5+
                (*mom).c6*(*fr).c6+(*mom).c7*(*fr).c7+(*mom).c8*(*fr).c8);
      mom+=1;
   }

   return sm;
}


int main(int argc,char *argv[])
{
   int my_rank,n,isw,status,ifail;
   int stat_min,stat_max;
   int bc,bs_hmc[4];
   int N0,N1,N2,nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf;
   int bs_sap[4],nmr,ncy,nkv,i0,i1,i2,i3;
   double beta,kappa,csw;
   double tau,res_acf,res_gcr_acf,res_rop,res_frf,res_gcr_frf;
   double eps,dsdt,act0,act1,dact;
   double dev_frc,sig_loss;
   complex_dble z;
   spinor_dble **wsd;   
   lat_parms_t lat;
   gcr_parms_t gcr;
   hmc_parms_t hmc;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check10.log","w",stdout);
      in=freopen("check10.in","r",stdin);
      
      printf("\n");
      printf("Check of hmc_gcr_vec(), hmc_gcr_frf() and hmc_gcr_acf()\n");
      printf("-------------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("kappa %lf csw %lf\n",&kappa,&csw);
      scanf("bc %d bs_hmc %d %d %d %d\n",
            &bc,&bs_hmc[0],&bs_hmc[1],&bs_hmc[2],&bs_hmc[3]);
      scanf("nmx_rop %d res_rop %lf\n",&nmx_rop,&res_rop);
      scanf("bs_sap %d %d %d %d\n",
            &bs_sap[0],&bs_sap[1],&bs_sap[2],&bs_sap[3]);
      scanf("nmr %d ncy %d nkv %d\n",&nmr,&ncy,&nkv);
      scanf("nmx_gcr_acf %d res_gcr_acf %lf\n",&nmx_gcr_acf,&res_gcr_acf);
      scanf("nmx_gcr_frf %d res_gcr_frf %lf",&nmx_gcr_frf,&res_gcr_frf);
      fclose(in);

      printf("Random gauge field\n");      
   }

   MPI_Bcast(&kappa,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&csw,1,MPI_DOUBLE,0,MPI_COMM_WORLD);   
   MPI_Bcast(&bc,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(bs_hmc,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_rop,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_rop,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(bs_sap,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmr,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&ncy,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nkv,1,MPI_INT,0,MPI_COMM_WORLD);   
   MPI_Bcast(&nmx_gcr_acf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_gcr_acf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_gcr_frf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_gcr_frf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   
   start_ranlux(0,1234);
   geometry();
   alloc_ud();
   alloc_s(2*nkv+1);
   alloc_sd(7);

   beta=0.0;
   N0=1;
   N1=1;
   N2=1;
   nmx_acf=1;
   nmx_frf=1;
   tau=0.0;
   res_acf=1.0;
   res_frf=1.0;

   lat=set_lat_parms(beta,kappa,csw);
   gcr=set_gcr_parms(bs_sap,nmr,ncy,nkv);
   hmc=set_hmc_parms(bs_hmc,bc,N0,N1,N2,
                     nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf,
                     tau,
                     res_acf,res_gcr_acf,res_rop,res_frf,res_gcr_frf);

   if (my_rank==0)
   {
      printf("kappa = %.6f\n",lat.kappa);      
      printf("csw = %.6f\n",lat.csw);
      printf("bc = %d\n",hmc.bc);            
      printf("bs_hmc = %d %d %d %d\n",bs_hmc[0],bs_hmc[1],bs_hmc[2],bs_hmc[3]);
      printf("nmx_rop = %d\n",hmc.nmx_rop);            
      printf("res_rop = %.2e\n",hmc.res_rop);      
      printf("bs_sap = %d %d %d %d\n",bs_sap[0],bs_sap[1],bs_sap[2],bs_sap[3]);
      printf("nmr = %d\n",gcr.nmr);                  
      printf("ncy = %d\n",gcr.ncy);
      printf("nkv = %d\n",gcr.nkv);                        
      printf("nmx_gcr_acf = %d\n",hmc.nmx_gcr_acf);            
      printf("res_gcr_acf = %.2e\n",hmc.res_gcr_acf);
      printf("nmx_gcr_frf = %d\n",hmc.nmx_gcr_frf);            
      printf("res_gcr_frf = %.2e\n\n",hmc.res_gcr_frf);

   }

   wsd=reserve_wsd(4);
   i0=(wsd[0]-psd[0][0])/NSPIN;
   i1=(wsd[1]-psd[0][0])/NSPIN;
   i2=(wsd[2]-psd[0][0])/NSPIN;
   i3=(wsd[3]-psd[0][0])/NSPIN;
   
   init_ud();
   if (bc==-1)
     flipbcd(); 
   assign_ud2udbgr(HMC_BLOCKS);
   swb_term();
   
   for (n=0;n<hmc.nb;n++)
   {
      init_mom(n);
      rot_ud(n,0.1);
   }

   random_chi();
   assign_chi2sd(i0);
   act0=norm_square_dble(VOLUME,1,wsd[0]);
   ROp(&status,&ifail);

   if (my_rank==0)
   {
      printf("Initialization of the pseudo-fermion field\n");
      printf("Application of ROp(): status = %d, ifail = %d\n\n",
             status,ifail);
   }

   hmc_gcr_vec(i1,i2,&status,&ifail);

   assign_chi2sd(i3);
   sub_assign_PdOs(i3,i1);
   z.re=-1.0;
   z.im=0.0;
   mulc_spinor_add_dble(VOLUME,wsd[i3],wsd[i0],z);
   eps=norm_square_dble(VOLUME,1,wsd[i3])/act0;
      
   if (my_rank==0)
   {
      printf("Application of hmc_gcr_vec():  status = %d, ifail = %d\n",
             status,ifail);
      printf("Check of first vector  = %.2e\n",sqrt(eps));
   }

   swap_active(HMC_BLOCKS);
   sw_term();
   Qnohat_dble(i2,i3);
   swap_active(HMC_BLOCKS);
   mulc_spinor_add_dble(VOLUME,wsd[i3],wsd[i0],z);
   eps=norm_square_dble(VOLUME,1,wsd[i3])/act0;

   if (my_rank==0)
      printf("Check of second vector = %.2e\n\n",sqrt(eps));
   
   act1=hmc_gcr_acf(i1,&status,&ifail);

   if (my_rank==0)
   {
      printf("hmc_gcr_acf() using good vector:  status = %d, ifail = %d\n",
             status,ifail);
      printf("Relative deviation of the action = %.2e\n",fabs(1.0-act1/act0));
   }

   act1=hmc_gcr_acf(i2,&status,&ifail);

   if (my_rank==0)
   {
      printf("hmc_gcr_acf() using false vector:  status = %d, ifail = %d\n",
             status,ifail);
      printf("Relative deviation of the action = %.2e\n",fabs(1.0-act1/act0));
   }

   eps=hmc.res_gcr_acf;
   hmc=set_hmc_parms(bs_hmc,bc,N0,N1,N2,
                     nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf,
                     tau,
                     res_acf,10.0*res_gcr_frf,res_rop,res_frf,res_gcr_frf);
   
   act1=hmc_gcr_acf(i1,&status,&ifail);
   hmc=set_hmc_parms(bs_hmc,bc,N0,N1,N2,
                     nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf,
                     tau,
                     res_acf,eps,res_rop,res_frf,res_gcr_frf);
   error_chk();
   
   if (my_rank==0)
   {
      printf("hmc_gcr_acf() with low precision:  status = %d, ifail = %d\n",
             status,ifail);
      printf("Relative deviation of the action = %.2e\n\n",fabs(1.0-act1/act0));
   }

   dact=0.0;

   for (n=0;n<hmc.nb;n++)
   {
      hmc_gcr_frf(n,i1,i2);
      dact+=dSdt(n);
   }

   MPI_Reduce(&dact,&dsdt,1,MPI_DOUBLE,MPI_SUM,0,MPI_COMM_WORLD);
   MPI_Bcast(&dsdt,1,MPI_DOUBLE,0,MPI_COMM_WORLD);

   ifail=0;
   stat_max=0;
   stat_min=0;
   eps=1.0e-3;
   
   for (n=0;n<hmc.nb;n++)
      rot_ud(n,eps);

   act0=2.0*hmc_gcr_acf(i1,&status,&isw)/3.0;

   for (n=0;n<hmc.nb;n++)
      rot_ud(n,-eps);

   ifail|=isw;
   if (status>stat_max)
      stat_max=status;
   if ((stat_min==0)||(status<stat_min))
      stat_min=status;

   for (n=0;n<hmc.nb;n++)
      rot_ud(n,-eps);

   act1=2.0*hmc_gcr_acf(i1,&status,&isw)/3.0;

   for (n=0;n<hmc.nb;n++)
      rot_ud(n,eps);

   ifail|=isw;
   if (status>stat_max)
      stat_max=status;
   if ((stat_min==0)||(status<stat_min))
      stat_min=status;

   for (n=0;n<hmc.nb;n++)
      rot_ud(n,2.0*eps);

   act0-=hmc_gcr_acf(i1,&status,&isw)/12.0;

   for (n=0;n<hmc.nb;n++)
      rot_ud(n,-2.0*eps);

   ifail|=isw;
   if (status>stat_max)
      stat_max=status;
   if ((stat_min==0)||(status<stat_min))
      stat_min=status;

   for (n=0;n<hmc.nb;n++)
      rot_ud(n,-2.0*eps);

   act1-=hmc_gcr_acf(i1,&status,&isw)/12.0;

   for (n=0;n<hmc.nb;n++)
      rot_ud(n,2.0*eps);

   ifail|=isw;
   if (status>stat_max)
      stat_max=status;
   if ((stat_min==0)||(status<stat_min))
      stat_min=status;
      
   dact=(act0-act1)/eps;
   dev_frc=fabs(1.0-dsdt/dact);

   sig_loss=-log10(fabs(1.0-act0/act1));

   error_chk();
   MPI_Reduce(&stat_max,&status,1,MPI_INT,MPI_MAX,0,MPI_COMM_WORLD);
   stat_max=status;
   MPI_Reduce(&stat_min,&status,1,MPI_INT,MPI_MIN,0,MPI_COMM_WORLD);
   stat_min=status;
   MPI_Reduce(&ifail,&isw,1,MPI_INT,MPI_MAX,0,MPI_COMM_WORLD);
   ifail=isw;

   if (my_rank==0)
   {
      printf("Calculation of the force:\n");
      printf("Relative deviation of dS/dt = %.2e ",dev_frc);
      printf("[significance loss = %d digits]\n",(int)(sig_loss));
      printf("status             = %d ... %d, ifail = %d\n\n",
             stat_min,stat_max,ifail);
      fclose(log);
   }
   
   MPI_Finalize();    
   exit(0);
}
