
/*******************************************************************************
*
* File check12.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Conservation of the action by the HMD evolution
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "block.h"
#include "sw_term.h"
#include "dirac.h"
#include "hmc.h"
#include "global.h"

static const su3_dble ud0={{0.0}};
static su3_alg_dble *msv=NULL;


static void init_ud(void)
{
   su3_alg_dble X;
   su3_dble *u,*um;

   u=pud[VOLUME/2][0];
   um=u+4*VOLUME;

   for (;u<um;u++)
   {
      *u=ud0;
      (*u).c11.re=1.0;
      (*u).c22.re=1.0;
      (*u).c33.re=1.0;      
      random_alg(1,&X);
      ExpX(0.1,&X,u);      
   }

   set_flags(NEW_UD);
   renormalize_ud();
}


static void save_mom(void)
{
   int nb,isw;
   su3_alg_dble *mom,*momm,*momsv;
   hmc_t *hflds,*hfldsm;

   blk_list(HMC_BLOCKS,&nb,&isw);
   hflds=hmcflds();
   hfldsm=hflds+nb;

   if (msv==NULL)
   {
      msv=amalloc(nb*(*hflds).nlks*sizeof(su3_alg_dble),ALIGN);
      error(msv==NULL,1,"save_mom [check12.c]",
            "Unable to allocate auxiliary array");
   }

   momsv=msv;
   
   for (;hflds<hfldsm;hflds++)
   {
      mom=(*hflds).mom;
      momm=mom+(*hflds).nlks;

      for (;mom<momm;mom++)
      {
         *momsv=*mom;
         momsv+=1;
      }
   }
}


static void restore_mom(void)
{
   int nb,isw;
   su3_alg_dble *mom,*momm,*momsv;
   hmc_t *hflds,*hfldsm;

   blk_list(HMC_BLOCKS,&nb,&isw);
   hflds=hmcflds();
   hfldsm=hflds+nb;

   error(msv==NULL,1,"restore_mom [check12.c]",
         "The momenta must first be saved");
   momsv=msv;
   
   for (;hflds<hfldsm;hflds++)
   {
      mom=(*hflds).mom;
      momm=mom+(*hflds).nlks;

      for (;mom<momm;mom++)
      {
         *mom=*momsv;
         momsv+=1;
      }
   }
}


static void restart_hmd(int N0,int N1,int N2)
{
   int n,nb,isw;
   block_t *b;
   hmc_parms_t hmc;

   b=blk_list(HMC_BLOCKS,&nb,&isw);
   hmc=hmc_parms();
   hmc=set_hmc_parms((*b).bs,hmc.bc,N0,N1,N2,
                     hmc.nmx_acf,hmc.nmx_gcr_acf,hmc.nmx_rop,
                     hmc.nmx_frf,hmc.nmx_gcr_frf,
                     hmc.tau,
                     hmc.res_acf,hmc.res_gcr_acf,hmc.res_rop,
                     hmc.res_frf,hmc.res_gcr_frf);
   
   n=query_flags(BCD_FLIPPED);
   if (((n!=0)&&(hmc.bc==1))||((n!=1)&&(hmc.bc==-1)))
      flipbcd();

   restore_mom();
   assign_ud2udbgr(HMC_BLOCKS);
   swb_term();
}


int main(int argc,char *argv[])
{
   int my_rank,i;
   int bs_hmc[4],bs_sap[4];
   int bc,N0,N1,N2;
   int nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf;
   int nkv,nmr,ncy;
   int stat_acf,stat_gcr_acf;
   int stat_rop,stat_frf,stat_gcr_frf,ifail;
   double beta,kappa,csw,tau;
   double res_acf,res_gcr_acf,res_rop,res_frf,res_gcr_frf;
   double act0,act1[3];
   lat_parms_t lat;
   gcr_parms_t gcr;
   hmc_parms_t hmc;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check12.log","w",stdout);
      in=freopen("check12.in","r",stdin);
      
      printf("\n");
      printf("Conservation of the action by the HMD evolution\n");
      printf("-----------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("beta %lf kappa %lf csw %lf bc %d\n",&beta,&kappa,&csw,&bc);
      scanf("bs_hmc %d %d %d %d\n",&bs_hmc[0],&bs_hmc[1],&bs_hmc[2],&bs_hmc[3]);
      scanf("tau %lf N0 %d N1 %d N2 %d\n",&tau,&N0,&N1,&N2);
      scanf("nmx_acf %d res_acf %lf\n",&nmx_acf,&res_acf);
      scanf("nmx_rop %d res_rop %lf\n",&nmx_rop,&res_rop);
      scanf("nmx_frf %d res_frf %lf\n",&nmx_frf,&res_frf);
      scanf("bs_sap %d %d %d %d\n",&bs_sap[0],&bs_sap[1],&bs_sap[2],&bs_sap[3]);
      scanf("nmr %d ncy %d nkv %d\n",&nmr,&ncy,&nkv);
      scanf("nmx_gcr_acf %d res_gcr_acf %lf\n",&nmx_gcr_acf,&res_gcr_acf);
      scanf("nmx_gcr_frf %d res_gcr_frf %lf",&nmx_gcr_frf,&res_gcr_frf);
      fclose(in);

      printf("Random gauge field\n");      
   }

   MPI_Bcast(&beta,1,MPI_DOUBLE,0,MPI_COMM_WORLD);   
   MPI_Bcast(&kappa,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&csw,1,MPI_DOUBLE,0,MPI_COMM_WORLD);   
   MPI_Bcast(&bc,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(bs_hmc,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&tau,1,MPI_DOUBLE,0,MPI_COMM_WORLD);   
   MPI_Bcast(&N0,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&N1,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&N2,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_acf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_acf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_rop,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_rop,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_frf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_frf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(bs_sap,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmr,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&ncy,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nkv,1,MPI_INT,0,MPI_COMM_WORLD);   
   MPI_Bcast(&nmx_gcr_acf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_gcr_acf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_gcr_frf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_gcr_frf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   
   start_ranlux(0,1234);
   geometry();
   alloc_ud();
   alloc_s(2*nkv+1);
   alloc_sd(4);

   lat=set_lat_parms(beta,kappa,csw);
   gcr=set_gcr_parms(bs_sap,nmr,ncy,nkv);
   hmc=set_hmc_parms(bs_hmc,bc,N0,N1,N2,
                     nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf,
                     tau,
                     res_acf,res_gcr_acf,res_rop,res_frf,res_gcr_frf);

   if (my_rank==0)
   {
      printf("beta = %.6f\n",lat.beta);      
      printf("kappa = %.6f\n",lat.kappa);      
      printf("csw = %.6f\n",lat.csw);
      printf("bc = %d\n",hmc.bc);            
      printf("bs_hmc = %d %d %d %d\n",bs_hmc[0],bs_hmc[1],bs_hmc[2],bs_hmc[3]);
      printf("N0 = %d\n",hmc.N0);
      printf("N1 = %d\n",hmc.N1);
      printf("N2 = %d\n",hmc.N2);      
      printf("tau = %.2f\n",hmc.tau);
      printf("nmx_acf = %d\n",hmc.nmx_acf);            
      printf("res_acf = %.2e\n",hmc.res_acf);      
      printf("nmx_rop = %d\n",hmc.nmx_rop);            
      printf("res_rop = %.2e\n",hmc.res_rop);      
      printf("nmx_frf = %d\n",hmc.nmx_frf);            
      printf("res_frf = %.2e\n",hmc.res_frf);      
      printf("bs_sap = %d %d %d %d\n",bs_sap[0],bs_sap[1],bs_sap[2],bs_sap[3]);
      printf("nmr = %d\n",gcr.nmr);                  
      printf("ncy = %d\n",gcr.ncy);
      printf("nkv = %d\n",gcr.nkv);                        
      printf("nmx_gcr_acf = %d\n",hmc.nmx_gcr_acf);            
      printf("res_gcr_acf = %.2e\n",hmc.res_gcr_acf);
      printf("nmx_gcr_frf = %d\n",hmc.nmx_gcr_frf);            
      printf("res_gcr_frf = %.2e\n\n",hmc.res_gcr_frf);
      fflush(log);
   }

   init_ud();
   act0=start_hmd(&stat_rop,&ifail);
   save_mom();

   if (my_rank==0)
   {
      printf("start_hmd:\n");
      printf("stat_rop = %d, ifail = %d\n",stat_rop,ifail);
      printf("action = % .4e\n\n",act0);
      fflush(log);
   }

   error_root((stat_rop<0)||(ifail!=0),1,"main [check12.c]",
              "start_hmd() failed");

   for (i=0;i<3;i++)
   {
      if (i>0)
         restart_hmd(hmc.N0,hmc.N1,hmc.N2<<i);
      act1[i]=run_hmd(&stat_acf,&stat_gcr_acf,&stat_frf,&stat_gcr_frf,&ifail);

      if (my_rank==0)
      {
         printf("run_hmd:\n");
         printf("N0,N1,N2 = %d,%d,%d\n",hmc.N0,hmc.N1,hmc.N2<<i);
         printf("stat_acf = %d\n",stat_acf);
         printf("stat_gcr_acf = %d\n",stat_gcr_acf);      
         printf("stat_frf = %d\n",stat_frf);
         printf("stat_gcr_frf = %d, ifail = %d\n",stat_gcr_frf,ifail);
         printf("action = % .4e\n\n",act1[i]);      
         fflush(log);
      }

      error_root((stat_acf<0)||(stat_gcr_acf<0)||(stat_frf<0)||
                 (stat_gcr_frf<0)||(ifail!=0),1,"main [check12.c]",
                 "run_hmd() failed");
   }

   error_chk();
   
   if (my_rank==0)
   {
      act1[0]-=act0;
      printf("eps0=%.3f, Delta H = %.2e\n",
             hmc.tau/(double)(N0*N1*N2),act1[0]);

      for (i=1;i<3;i++)
      {
         act1[i]-=act0;
         printf("eps0=%.3f, Delta H = %.2e, |Delta H[i]/Delta H[i-1]| = %.2e\n",
                hmc.tau/(double)(N0*N1*(N2<<i)),act1[i],
                fabs(act1[i]/act1[i-1]));
      }
      fclose(log);
   }
      
   MPI_Finalize();    
   exit(0);
}
