
/*******************************************************************************
*
* File check13.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Statistical test of the pseudo-fermion distributions
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "block.h"
#include "sw_term.h"
#include "dirac.h"
#include "hmc.h"
#include "global.h"

static const su3_dble ud0={{0.0}};


static void init_ud(void)
{
   su3_alg_dble X;
   su3_dble *u,*um;

   u=pud[VOLUME/2][0];
   um=u+4*VOLUME;

   for (;u<um;u++)
   {
      *u=ud0;
      (*u).c11.re=1.0;
      (*u).c22.re=1.0;
      (*u).c33.re=1.0;      
      random_alg(1,&X);
      ExpX(0.1,&X,u);      
   }

   set_flags(NEW_UD);
   renormalize_ud();
}


int main(int argc,char *argv[])
{
   int my_rank,n,nb,isw,i0,v1,v2,*bs;
   int bs_hmc[4],bs_sap[4];
   int bc,N0,N1,N2;
   int nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf;
   int nkv,nmr,ncy;
   int stat,flag,stat_min,stat_sum;
   int stat_acf,stat_gcr_acf;
   int stat_rop,ifail;
   double beta,kappa,csw,tau;
   double res_acf,res_gcr_acf,res_rop,res_frf,res_gcr_frf;
   double act0,blk_act,act1;
   spinor_dble **wsd;
   lat_parms_t lat;
   gcr_parms_t gcr;
   hmc_parms_t hmc;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check13.log","w",stdout);
      in=freopen("check13.in","r",stdin);
      
      printf("\n");
      printf("Statistical test of the pseudo-fermion distributions\n");
      printf("----------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("kappa %lf csw %lf bc %d\n",&kappa,&csw,&bc);
      scanf("bs_hmc %d %d %d %d\n",&bs_hmc[0],&bs_hmc[1],&bs_hmc[2],&bs_hmc[3]);
      scanf("nmx_acf %d res_acf %lf\n",&nmx_acf,&res_acf);
      scanf("nmx_rop %d res_rop %lf\n",&nmx_rop,&res_rop);
      scanf("bs_sap %d %d %d %d\n",&bs_sap[0],&bs_sap[1],&bs_sap[2],&bs_sap[3]);
      scanf("nmr %d ncy %d nkv %d\n",&nmr,&ncy,&nkv);
      scanf("nmx_gcr_acf %d res_gcr_acf %lf\n",&nmx_gcr_acf,&res_gcr_acf);
      fclose(in);

      
      printf("Random gauge field\n");      
   }

   MPI_Bcast(&kappa,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&csw,1,MPI_DOUBLE,0,MPI_COMM_WORLD);   
   MPI_Bcast(&bc,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(bs_hmc,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_acf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_acf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_rop,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_rop,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(bs_sap,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmr,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&ncy,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nkv,1,MPI_INT,0,MPI_COMM_WORLD);   
   MPI_Bcast(&nmx_gcr_acf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_gcr_acf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);

   beta=0.0;
   tau=0.5;
   N0=1;
   N1=1;
   N2=1;
   nmx_frf=1;
   res_frf=1.0;
   nmx_gcr_frf=1;
   res_gcr_frf=1.0;

   start_ranlux(0,123);
   geometry();
   alloc_ud();
   alloc_s(2*nkv+1);
   alloc_sd(4);
   
   lat=set_lat_parms(beta,kappa,csw);
   gcr=set_gcr_parms(bs_sap,nmr,ncy,nkv);
   hmc=set_hmc_parms(bs_hmc,bc,N0,N1,N2,
                     nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf,
                     tau,
                     res_acf,res_gcr_acf,res_rop,res_frf,res_gcr_frf);

   if (my_rank==0)
   {
      printf("kappa = %.6f\n",lat.kappa);      
      printf("csw = %.6f\n",lat.csw);
      printf("bc = %d\n",hmc.bc);            
      printf("bs_hmc = %d %d %d %d\n",bs_hmc[0],bs_hmc[1],bs_hmc[2],bs_hmc[3]);
      printf("nmx_acf = %d\n",hmc.nmx_acf);            
      printf("res_acf = %.2e\n",hmc.res_acf);      
      printf("nmx_rop = %d\n",hmc.nmx_rop);            
      printf("res_rop = %.2e\n",hmc.res_rop);      
      printf("bs_sap = %d %d %d %d\n",bs_sap[0],bs_sap[1],bs_sap[2],bs_sap[3]);
      printf("nmr = %d\n",gcr.nmr);                  
      printf("ncy = %d\n",gcr.ncy);
      printf("nkv = %d\n",gcr.nkv);                        
      printf("nmx_gcr_acf = %d\n",hmc.nmx_gcr_acf);            
      printf("res_gcr_acf = %.2e\n\n",hmc.res_gcr_acf);
      fflush(log);
   }

   init_ud();
   act0=start_hmd(&stat_rop,&ifail);

   if (my_rank==0)
   {
      printf("start_hmd:\n");
      printf("stat_rop = %d, ifail = %d\n",stat_rop,ifail);
      printf("action = % .4e\n\n",act0);
      fflush(log);
   }

   error_root((stat_rop<0)||(ifail!=0),1,"main [check13.c]",
              "start_hmd() failed");

   blk_list(HMC_BLOCKS,&nb,&isw);

   blk_act=0.0;
   stat_acf=0;
   ifail=0;

   for (n=0;n<nb;n++)
   {
      blk_act+=hmc_acf(n,1,0,&stat,&flag);

      if (stat<0)
         stat_acf=stat;
      else
         stat_acf+=stat;

      ifail|=flag;
      
      if ((stat_acf<0)||(ifail!=0))
         break;
   }
                        
   MPI_Reduce(&stat_acf,&stat_min,1,MPI_INT,MPI_MIN,0,MPI_COMM_WORLD);
   MPI_Reduce(&stat_acf,&stat_sum,1,MPI_INT,MPI_SUM,0,MPI_COMM_WORLD);
   MPI_Reduce(&ifail,&flag,1,MPI_INT,MPI_MAX,0,MPI_COMM_WORLD);
   MPI_Bcast(&stat_min,1,MPI_INT,0,MPI_COMM_WORLD);      
   MPI_Bcast(&stat_sum,1,MPI_INT,0,MPI_COMM_WORLD);            
   MPI_Bcast(&flag,1,MPI_INT,0,MPI_COMM_WORLD);

   if (stat_min<0)
      stat_acf=stat_min;
   else
      stat_acf=stat_sum/(NPROC*nb);
   
   ifail=flag;

   error_root((stat_acf<0)||(ifail!=0),1,"main [check13.c]",
              "Calculation of block action failed");

   MPI_Reduce(&blk_act,&act1,1,MPI_DOUBLE,MPI_SUM,0,MPI_COMM_WORLD);
   MPI_Bcast(&act1,1,MPI_DOUBLE,0,MPI_COMM_WORLD);

   n=6*NPROC*VOLUME;
   
   if (my_rank==0)
   {
      printf("Expectation value of the block pseudo-fermion action:\n");
      printf("stat_acf = %d, ifail = %d\n",stat_acf,ifail);
      printf("<action>/point = %.4f ",act1/(double)(n));
      printf("(expect 1.0000+-%.4f)\n\n",sqrt(1.0/(double)(2*n)));
      fflush(log);
   }

   wsd=reserve_wsd(1);
   i0=(wsd[0]-psd[0][0])/NSPIN;
   
   act1=hmc_gcr_acf(i0,&stat_gcr_acf,&ifail);

   error_root((stat_gcr_acf<0)||(ifail!=0),1,"main [check13.c]",
              "Calculation of block-interaction action failed");
   error_chk();
   
   bs=bs_hmc;

   v2=2*((bs[0]-2)*(bs[1]-2)*(bs[2]-2)+
         (bs[0]-2)*(bs[1]-2)*(bs[3]-2)+
         (bs[0]-2)*(bs[2]-2)*(bs[3]-2)+
         (bs[1]-2)*(bs[2]-2)*(bs[3]-2));
   
   v1=bs[0]*bs[1]*bs[2]*bs[3]-(bs[0]-2)*(bs[1]-2)*(bs[2]-2)*(bs[3]-2)-v2;
   n=((12*v1+6*v2)*(NPROC*nb))/2;
   
   if (my_rank==0)
   {
      printf("Expectation value of the block-interaction action:\n");
      printf("stat_gcr_acf = %d, ifail = %d\n",stat_acf,ifail);
      printf("<action>/point = %.4f ",act1/(double)(n));
      printf("(expect 1.0000+-%.4f)\n\n",sqrt(1.0/(double)(2*n)));
      fclose(log);      
   }
   
   MPI_Finalize();    
   exit(0);
}
