
/*******************************************************************************
*
* File check14.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the acceptance step
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "block.h"
#include "linalg.h"
#include "dirac.h"
#include "sw_term.h"
#include "hmc.h"
#include "global.h"

static const su3_dble ud0={{0.0}};


static void init_ud(void)
{
   su3_alg_dble X;
   su3_dble *u,*um;

   u=pud[VOLUME/2][0];
   um=u+4*VOLUME;

   for (;u<um;u++)
   {
      *u=ud0;
      (*u).c11.re=1.0;
      (*u).c22.re=1.0;
      (*u).c33.re=1.0;      
      random_alg(1,&X);
      ExpX(0.1,&X,u);      
   }

   set_flags(NEW_UD);
   renormalize_ud();
}


static void rot_ud(int n,double t)
{
   int nb,isw,*lk,*lkm;
   su3_dble *ub;
   su3_alg_dble *mom;
   block_t *b;
   hmc_t *hflds;

   b=blk_list(HMC_BLOCKS,&nb,&isw)+n;
   hflds=hmcflds()+n;
   
   ub=(*b).ud[0];
   mom=(*hflds).mom;
   lk=(*hflds).lks;
   lkm=lk+(*hflds).nlks;
   
   for (;lk<lkm;lk++)
   {
      ExpX(t,mom,ub+(*lk));
      mom+=1;
   }

   set_blk_flags(HMC_BLOCKS,NEW_ACTIVE_UD,n);
}


static double action(void)
{
   int n,nb,isw;
   double blk_act;

   blk_act=0.0;
   blk_list(HMC_BLOCKS,&nb,&isw);

   for (n=0;n<nb;n++)
      blk_act+=hmc_acg(n);

   return blk_act;
}


int main(int argc,char *argv[])
{
   int my_rank,n,nb,isw;
   int bc,bs[4],Ntry,ib,iac;
   double act[2],act1,dH,Pacc,p;
   block_t *b;
   hmc_t *hflds;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check14.log","w",stdout);
      in=freopen("check14.in","r",stdin);
      
      printf("\n");
      printf("Check of the acceptance step\n");
      printf("----------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bc %d bs %d %d %d %d\n",&bc,&bs[0],&bs[1],&bs[2],&bs[3]);
      scanf("dH %lf Ntry %d",&dH,&Ntry);
      fclose(in);

      printf("bc = %d\n",bc);
      printf("bs = %d %d %d %d\n",bs[0],bs[1],bs[2],bs[3]);
      printf("Number of trials = %d\n\n",Ntry);
   }

   MPI_Bcast(&bc,1,MPI_INT,0,MPI_COMM_WORLD);   
   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&dH,1,MPI_DOUBLE,0,MPI_COMM_WORLD);      
   MPI_Bcast(&Ntry,1,MPI_INT,0,MPI_COMM_WORLD);   
   
   start_ranlux(0,123);
   geometry();
   alloc_ud();
   alloc_bgr(HMC_BLOCKS,bs);
   b=blk_list(HMC_BLOCKS,&nb,&isw);

   Pacc=0.0;

   for (n=0;n<Ntry;n++)
   {
      init_ud();
      if (bc==-1)
         flipbcd();
      assign_ud2udbgr(HMC_BLOCKS);

      hflds=hmcflds();
   
      for (ib=0;ib<nb;ib++)
      {
         random_alg((*hflds).nlks,(*hflds).mom);
         rot_ud(ib,0.1);
         hflds+=1;
      }

      act[1]=action();
      swap_active(HMC_BLOCKS);
      act[0]=action();

      MPI_Reduce(&act[1],&act1,1,MPI_DOUBLE,MPI_SUM,0,MPI_COMM_WORLD);
      MPI_Bcast(&act1,1,MPI_DOUBLE,0,MPI_COMM_WORLD);      

      iac=accept_step(act1,act1+dH);
      Pacc+=(double)(iac);

      error_loc(act[iac]!=action(),1,"main [check14.c]",
                "Fields swapping is incorrect");
      error(query_flags(BCD_FLIPPED)!=0,1,"main [check14.c]",
            "Boundary links are flipped");
   }

   Pacc/=(double)(Ntry);

   if (dH>0.0)
      p=exp(-dH);
   else
      p=1.0;
   
   error_chk();
 
   if (my_rank==0)
   {
      printf("No errors discovered\n");
      printf("Acceptance rate = %.3f ",Pacc);
      printf("(expect %.3f+-%.3f)\n\n",p,sqrt(p*(1.0-p)/(double)(Ntry)));
      fclose(log);   
   }      
   
   MPI_Finalize();   
   exit(0);
}
