
/*******************************************************************************
*
* File check2.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the "exponential" function E(t*X)
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "eva.h"
#include "hmc.h"
#include "global.h"

#define NMOM 2500

static double ta[5],del[5],delm[5];
static const su3_dble u0={{0.0}};


static void dev(su3_dble *u,double *d1,double *d2)
{
   int i;
   double d;
   complex_dble det1,det2,det3,det;
   su3_dble v,w;

   _su3_dagger(v,(*u));
   _su3_times_su3(w,v,(*u));

   w.c11.re-=1.0;
   w.c22.re-=1.0;
   w.c33.re-=1.0;

   *d1=0.0;

   for (i=0;i<18;i++)
   {
      d=*((double*)(&w)+i);
      d=fabs(d);
      if (d>(*d1))
         *d1=d;
   }

   det1.re=
      ((*u).c22.re*(*u).c33.re-(*u).c22.im*(*u).c33.im)-
      ((*u).c23.re*(*u).c32.re-(*u).c23.im*(*u).c32.im);
   det1.im=
      ((*u).c22.re*(*u).c33.im+(*u).c22.im*(*u).c33.re)-
      ((*u).c23.re*(*u).c32.im+(*u).c23.im*(*u).c32.re);
   det2.re=
      ((*u).c21.re*(*u).c33.re-(*u).c21.im*(*u).c33.im)-
      ((*u).c23.re*(*u).c31.re-(*u).c23.im*(*u).c31.im);
   det2.im=
      ((*u).c21.re*(*u).c33.im+(*u).c21.im*(*u).c33.re)-
      ((*u).c23.re*(*u).c31.im+(*u).c23.im*(*u).c31.re);    
   det3.re=
      ((*u).c21.re*(*u).c32.re-(*u).c21.im*(*u).c32.im)-
      ((*u).c22.re*(*u).c31.re-(*u).c22.im*(*u).c31.im);
   det3.im=
      ((*u).c21.re*(*u).c32.im+(*u).c21.im*(*u).c32.re)-
      ((*u).c22.re*(*u).c31.im+(*u).c22.im*(*u).c31.re);

   det.re=
      ((*u).c11.re*det1.re-(*u).c11.im*det1.im)-
      ((*u).c12.re*det2.re-(*u).c12.im*det2.im)+
      ((*u).c13.re*det3.re-(*u).c13.im*det3.im);
   det.im=
      ((*u).c11.re*det1.im+(*u).c11.im*det1.re)-
      ((*u).c12.re*det2.im+(*u).c12.im*det2.re)+
      ((*u).c13.re*det3.im+(*u).c13.im*det3.re);

   *d2=0.0;
   d=fabs(det.re-1.0);
   if (d>(*d2))
      *d2=d;
   d=fabs(det.im);
   if (d>(*d2))
      *d2=d;
}


static su3_dble expX(double t,su3_alg_dble *X)
{
   double cd[3];
   su3_dble a,u,v,w;
   complex_dble *ca,*cv;

   ca=(complex_dble*)(&a);
   cv=(complex_dble*)(&v);
   
   a.c11.re=(*X).c1+(*X).c2;   
   a.c11.im=0.0;
   a.c22.re=(*X).c2-2.0*(*X).c1;
   a.c22.im=0.0;
   a.c33.re=(*X).c1-2.0*(*X).c2;
   a.c33.im=0.0;

   a.c12.re= (*X).c4;         
   a.c12.im=-(*X).c3;
   a.c13.re= (*X).c6;      
   a.c13.im=-(*X).c5;
   a.c23.re= (*X).c8;     
   a.c23.im=-(*X).c7;

   a.c21.re= a.c12.re;
   a.c21.im=-a.c12.im;
   a.c31.re= a.c13.re;
   a.c31.im=-a.c13.im;
   a.c32.re= a.c23.re;
   a.c32.im=-a.c23.im;
   
   jacobi2_dble(3,ca,cd,cv);
   
   w=u0;
   w.c11.re=cos(t*cd[0]);
   w.c11.im=sin(t*cd[0]);
   w.c22.re=cos(t*cd[1]);
   w.c22.im=sin(t*cd[1]);   
   w.c33.re=cos(t*cd[2]);
   w.c33.im=sin(t*cd[2]);   

   _su3_dagger(u,v);
   _su3_times_su3(a,w,u);
   _su3_times_su3(w,v,a);   
   
   return w;
}


static su3_dble dev_exp(double t,su3_dble *u,su3_alg_dble *X)
{
   su3_dble a,b;

   a=expX(-t,X);
   _su3_times_su3(b,(*u),a);

   b.c11.re-=1.0;
   b.c22.re-=1.0;   
   b.c33.re-=1.0;

   return b;
}


int main(int argc,char *argv[])
{
   int my_rank,n,i,j;
   double *r,*s;
   su3_dble *M,u,v;
   su3_alg_dble *X;
   FILE *log=NULL;   

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check2.log","w",stdout);
   
      printf("\n");
      printf("Check of the `exponential' function E(t*X)\n");
      printf("------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      printf("Number of random matrices considered: %d\n",NPROC*NMOM);
      printf("Distribution of X proportional to exp(tr{X^2})\n\n");
   }
   
   start_ranlux(0,123456);   

   X=amalloc(NMOM*sizeof(su3_alg_dble),4);
   M=amalloc(NMOM*sizeof(su3_dble),4);   
   error((X==NULL)||(M==NULL),1,
         "main [check2.c]","Unable to allocate field arrays");   

   random_alg(NMOM,X);

   for (n=0;n<NMOM;n++)
   {
      M[n]=u0;
      M[n].c11.re=1.0;
      M[n].c22.re=1.0;
      M[n].c33.re=1.0;

      ExpX(1.0,X+n,M+n);
   }

   random_alg(NMOM,X);   
   delm[0]=0.0;
   delm[1]=0.0;
   
   for (n=0;n<NMOM;n++)
   {
      ExpX(2.7819,X+n,M+n);
      dev(M+n,&del[0],&del[1]);      

      if (del[0]>delm[0])
         delm[0]=del[0];
      if (del[1]>delm[1])
         delm[1]=del[1];
   }
   
   MPI_Reduce(delm,del,2,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);

   if (my_rank==0)
   {   
      printf("Check that E(t*X)*u is in SU(3):\n");
      printf("Maximal deviation from unitarity     = %.2e\n",del[0]);
      printf("Maximal deviation from unimodularity = %.2e\n\n",del[1]);
   }

   random_alg(NMOM,X);
   delm[0]=0.0;

   for (n=0;n<NMOM;n++)
   {
      u=M[n];
      v=u;

      ExpX( 0.87612,X+n,&u);
      ExpX(-0.87612,X+n,&u);

      r=(double*)(&u);
      s=(double*)(&v);

      for (i=0;i<18;i++)
      {
         del[0]=fabs(r[i]-s[i]);
         if (del[0]>delm[0])
            delm[0]=del[0];
      }
   }

   MPI_Reduce(delm,del,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);

   if (my_rank==0)
   {   
      printf("Check that E(-t*X)*E(t*X)=1:\n");
      printf("Maximal deviation = %.2e\n\n",del[0]);
   }

   random_alg(NMOM,X);

   ta[0]=1.0;
   ta[1]=0.1;
   ta[2]=0.025;
   ta[3]=0.001;
   ta[4]=0.0001;

   delm[0]=0.0;
   delm[1]=0.0;
   delm[2]=0.0;
   delm[3]=0.0;   
   delm[4]=0.0;   
   
   for (n=0;n<NMOM;n++)
   {
      for (i=0;i<5;i++)
      {
         _su3_dagger(u,M[n]);         
         ExpX(ta[i],X+n,M+n);
         _su3_times_su3(v,M[n],u);
         u=dev_exp(ta[i],&v,X+n);

         r=(double*)(&u);

         for (j=0;j<18;j++)
         {
            del[i]=fabs(r[j]);
            if (del[i]>delm[i])
               delm[i]=del[i];
         }
      }
   }

   error_chk();
   
   MPI_Reduce(delm,del,5,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);

   if (my_rank==0)
   {   
      printf("Deviation of E(t*X)*exp(-t*X) from 1:\n");

      for (i=0;i<5;i++)
         printf("t = %.2e, maximal deviation = %.2e\n",ta[i],del[i]);

      printf("\n");
      fclose(log);
   }

   MPI_Finalize();    
   exit(0);
}
