
/*******************************************************************************
*
* File check9.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of ROp()
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "block.h"
#include "linalg.h"
#include "dirac.h"
#include "sw_term.h"
#include "sap_gcr.h"
#include "hmc.h"
#include "global.h"

static const su3_dble ud0={{0.0}};


static void init_ud(void)
{
   su3_alg_dble X;
   su3_dble *u,*um;

   u=pud[VOLUME/2][0];
   um=u+4*VOLUME;

   for (;u<um;u++)
   {
      *u=ud0;
      (*u).c11.re=1.0;
      (*u).c22.re=1.0;
      (*u).c33.re=1.0;      
      random_alg(1,&X);
      ExpX(0.1,&X,u);      
   }

   set_flags(NEW_UD);
   renormalize_ud();
}


int main(int argc,char *argv[])
{
   int my_rank,nb,isw,status,ifail;
   int bc,bs_hmc[4];
   int N0,N1,N2,nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf;
   int bs_sap[4],nmr,ncy,nkv,i0,i1,i2,i3,i4;
   double beta,kappa,csw;
   double tau,res_acf,res_gcr_acf,res_rop,res_frf,res_gcr_frf;
   double dmax,res,cnrm;
   complex_dble z;
   spinor_dble **wsd;
   block_t *b,*bm;
   lat_parms_t lat;
   hmc_parms_t hmc;
   gcr_parms_t gcr;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check9.log","w",stdout);
      in=freopen("check9.in","r",stdin);
      
      printf("\n");
      printf("Check of the program ROp\n");
      printf("------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("kappa %lf csw %lf\n",&kappa,&csw);
      scanf("bc %d bs_hmc %d %d %d %d\n",
            &bc,&bs_hmc[0],&bs_hmc[1],&bs_hmc[2],&bs_hmc[3]);
      scanf("nmx_rop %d res_rop %lf\n",&nmx_rop,&res_rop);
      scanf("bs_sap %d %d %d %d\n",
            &bs_sap[0],&bs_sap[1],&bs_sap[2],&bs_sap[3]);
      scanf("nmr %d ncy %d nkv %d\n",&nmr,&ncy,&nkv);
      scanf("nmx_gcr_acf %d res_gcr_acf %lf",&nmx_gcr_acf,&res_gcr_acf);
      fclose(in);

      printf("Random gauge field\n");
   }

   MPI_Bcast(&kappa,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(&csw,1,MPI_DOUBLE,0,MPI_COMM_WORLD);   
   MPI_Bcast(&bc,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(bs_hmc,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmx_rop,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_rop,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   MPI_Bcast(bs_sap,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmr,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&ncy,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nkv,1,MPI_INT,0,MPI_COMM_WORLD);   
   MPI_Bcast(&nmx_gcr_acf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&res_gcr_acf,1,MPI_DOUBLE,0,MPI_COMM_WORLD);

   start_ranlux(0,1234);
   geometry();
   alloc_ud();
   alloc_s(2*nkv+1);
   alloc_sd(6);
   
   beta=0.0;
   N0=1;
   N1=1;
   N2=1;
   nmx_acf=1;
   nmx_frf=1;
   nmx_gcr_frf=1;
   tau=0.0;
   res_acf=1.0;
   res_frf=1.0;
   res_gcr_frf=1.0;

   lat=set_lat_parms(beta,kappa,csw);
   gcr=set_gcr_parms(bs_sap,nmr,ncy,nkv);
   hmc=set_hmc_parms(bs_hmc,bc,N0,N1,N2,
                     nmx_acf,nmx_gcr_acf,nmx_rop,nmx_frf,nmx_gcr_frf,
                     tau,
                     res_acf,res_gcr_acf,res_rop,res_frf,res_gcr_frf);

   if (my_rank==0)
   {
      printf("kappa = %.6f\n",lat.kappa);      
      printf("csw = %.6f\n",lat.csw);
      printf("bc = %d\n",hmc.bc);            
      printf("bs_hmc = %d %d %d %d\n",bs_hmc[0],bs_hmc[1],bs_hmc[2],bs_hmc[3]);
      printf("nmx_rop = %d\n",hmc.nmx_rop);            
      printf("res_rop = %.2e\n",hmc.res_rop);      
      printf("bs_sap = %d %d %d %d\n",bs_sap[0],bs_sap[1],bs_sap[2],bs_sap[3]);
      printf("nmr = %d\n",gcr.nmr);                  
      printf("ncy = %d\n",gcr.ncy);
      printf("nkv = %d\n",gcr.nkv);                        
      printf("nmx_gcr_acf = %d\n",hmc.nmx_gcr_acf);            
      printf("res_gcr_acf = %.2e\n\n",hmc.res_gcr_acf);
   }

   init_ud();
   if (bc==-1)
      flipbcd();   
   assign_ud2udbgr(HMC_BLOCKS);
   swb_term();
   wsd=reserve_wsd(4);
   i0=(wsd[0]-psd[0][0])/NSPIN;
   i1=(wsd[1]-psd[0][0])/NSPIN;
   i2=(wsd[2]-psd[0][0])/NSPIN;
   i3=(wsd[3]-psd[0][0])/NSPIN;
   
   random_chi();
   assign_chi2sd(i0);
   cnrm=norm_square_dble(VOLUME,1,wsd[0]);
   ROp(&status,&ifail);

   if (my_rank==0)
      printf("Application of ROp: status = %d, ifail = %d\n\n",status,ifail);

   assign_chi2sd(i1);
   set_sd2zero(VOLUME,wsd[2]);
   b=blk_list(HMC_BLOCKS,&nb,&isw);
   bm=b+nb;

   for (;b<bm;b++)
   {
      assign_sd2sdb(b,ALL_PTS,i1,0);
      Qbnds_dble(b,0,0);
      sub_assign_sdbb2sd(b,0,i2);
   } 

   mulmg5_dble(VOLUME,wsd[2]);

   res=sap_gcr(hmc.nmx_gcr_acf,hmc.res_gcr_acf,i2,i3,&status,&ifail);

   release_wsd();
   wsd=reserve_wsd(5);
   i4=(wsd[4]-psd[0][0])/NSPIN;   
   
   Qnohat_dble(i3,i4);
   mulg5_dble(VOLUME,wsd[4]);
   z.re=-1.0;
   z.im=0.0;
   mulc_spinor_add_dble(VOLUME,wsd[4],wsd[2],z);
   dmax=norm_square_dble(VOLUME,1,wsd[4])/
      norm_square_dble(VOLUME,1,wsd[2]);   
   
   if (my_rank==0)
   {
      printf("Application of the SAP+GCR solver:\n");
      printf("Status = %d, ifail = %d, residue = %.1e\n\n",
             status,ifail,sqrt(dmax));
   }

   sub_assign_PdOs(1,3);
   mulc_spinor_add_dble(VOLUME,wsd[0],wsd[1],z);
   dmax=norm_square_dble(VOLUME,1,wsd[0])/cnrm;
   error_chk();
   
   if (my_rank==0)
   {
      printf("||chi-R^(-1)*ROp*chi||/||chi|| = %.1e\n\n",sqrt(dmax));
      fclose(log);
   }

   MPI_Finalize();   
   exit(0);
}
