
/*******************************************************************************
*
* File time1.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of the "exponential" function E(t*X)
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "hmc.h"
#include "global.h"

#define NMOM 100000

static const su3_dble ud0={{0.0}};


int main(int argc,char *argv[])
{
   int my_rank,n,k,count;
   double t,wt1,wt2,wdt;
   su3_dble *M;
   su3_alg_dble *X;
   FILE *log=NULL;   

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
   
   if (my_rank==0)
   {
      log=freopen("time1.log","w",stdout);
 
      printf("\n");
      printf("Timing of the `exponential' function E(t*X)\n");
      printf("-------------------------------------------\n\n");

      printf("%dx%dx%dx%d process grid\n",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("Number of momenta: %d\n\n",NMOM);      
   }

   start_ranlux(0,12345);

   X=amalloc(NMOM*sizeof(su3_alg_dble),4);
   M=amalloc(NMOM*sizeof(su3_dble),4);   
   error((X==NULL)||(M==NULL),1,
         "main [time1.c]","Unable to allocate field arrays");   

   random_alg(NMOM,X);

   for (k=0;k<NMOM;k++)
   {
      M[k]=ud0;
      M[k].c11.re=1.0;
      M[k].c22.re=1.0;
      M[k].c33.re=1.0;

      ExpX(1.0,X+k,M+k);
   }

   random_alg(NMOM,X);
   t=0.123;
   
   n=500000/NMOM;
   if (n<2)
      n=2;
   
   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();     
   for (count=0;count<n;count++)
   {
      t=-t;
      
      for (k=0;k<NMOM;k++)
         ExpX(t,X+k,M+k);
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   wdt=1.0e6*(wt2-wt1)/(double)(n*NMOM);
   error_chk();
   
   if (my_rank==0)
   {
      printf("Time per application of E(t*X): %4.3f micro sec\n\n",wdt);
      fclose(log);
   }   

   MPI_Finalize();   
   exit(0);
}
