
/*******************************************************************************
*
* File time3.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of sw_frc() and hop_frc()
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "block.h"
#include "linalg.h"
#include "dirac.h"
#include "sw_term.h"
#include "hmc.h"
#include "global.h"

static const su3_dble ud0={{0.0}};


static void init_ud(void)
{
   su3_alg_dble X;
   su3_dble *u,*um;

   u=pud[VOLUME/2][0];
   um=u+4*VOLUME;

   for (;u<um;u++)
   {
      *u=ud0;
      (*u).c11.re=1.0;
      (*u).c22.re=1.0;
      (*u).c33.re=1.0;      
      random_alg(1,&X);
      ExpX(0.1,&X,u);      
   }

   set_flags(NEW_UD);
   renormalize_ud();
}


int main(int argc,char *argv[])
{
   int my_rank,n,count,nb,isw,ib;
   int bs[4],ifail;
   double wt1,wt2,wdt;
   block_t *b;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
   
   if (my_rank==0)
   {
      log=freopen("time3.log","w",stdout);
      in=freopen("time3.in","r",stdin);
      
      printf("\n");
      printf("Timing of sw_frc() and hop_frc()\n");
      printf("--------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bs %d %d %d %d",&bs[0],&bs[1],&bs[2],&bs[3]);      
      fclose(in);

      printf("bs = %d %d %d %d\n\n",bs[0],bs[1],bs[2],bs[3]);
   }

   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);

   set_lat_parms(0.0,0.1,1.234);
   start_ranlux(0,1234);
   geometry();
   alloc_ud();
   alloc_bgr(HMC_BLOCKS,bs);

   init_ud();
   assign_ud2udbgr(HMC_BLOCKS);
   swb_term();
   b=blk_list(HMC_BLOCKS,&nb,&isw);
   random_sd((*b).vol,(*b).sd[0],1.0);
   random_sd((*b).vol,(*b).sd[1],1.0);   
   
   n=2000000/VOLUME;
   if (n<2)
      n=2;
   
   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();     
   for (count=0;count<n;count++)
   {
      for (ib=0;ib<nb;ib++)
         hop_frc(ib,0,1);
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);
   error_chk();
   
   if (my_rank==0)
   {
      printf("The timings per lattice point are:\n");
      printf("hop_frc:                  %4.3f micro sec\n",wdt);
   }   

   n/=10;
   if (n<2)
      n=2;

   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();     
   for (count=0;count<n;count++)
   {
      for (ib=0;ib<nb;ib++)
         sw_frc(ib,0,1,0);
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);
   error_chk();
   
   if (my_rank==0)
      printf("sw_frc w/o determinants:  %4.3f micro sec\n",wdt);

   ifail=assign_swd2swdbgr(HMC_BLOCKS,ALL_PTS);
   
   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();     
   for (count=0;count<n;count++)
   {
      for (ib=0;ib<nb;ib++)
         ifail|=sw_frc(ib,0,1,1);
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);
   error_chk();
   
   if (my_rank==0)
   {
      printf("sw_frc with determinants: %4.3f micro sec (ifail = %d)\n\n",
             wdt,ifail);
      printf("(not counting the required inversions of the SW-term)\n\n");
      fclose(log);
   }   
   
   MPI_Finalize();   
   exit(0);
}
