
/*******************************************************************************
*
* File check2.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Writing and reading configurations
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "misc.h"
#include "start.h"
#include "update.h"
#include "global.h"

static float rs1[100],rs2[100];
static double rd1[100],rd2[100];
static su3_dble u[4*VOLUME];


int main(int argc,char *argv[])
{
   int my_rank,i;
   float beta;
   double *ru,*rv,d,dmax;
   su3_dble *v;
   char name[80];
   FILE *log=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check2.log","w",stdout);

      printf("\n");
      printf("Writing and reading configurations\n");
      printf("----------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);
      fflush(log);
   }

   beta=5.7f;
   start_ranlux(0,123456);
   geometry();
   alloc_u();
   alloc_ud();
   random_u();

   update(beta,4,0,8);
   assign_u2ud();

   sprintf(name,"check2_%d.dat",my_rank);
   write_cnfg(name);

   ranlxs(rs1,100);
   ranlxd(rd1,100);
   v=pud[VOLUME/2][0];

   for (i=0;i<(4*VOLUME);i++)
      u[i]=v[i];

   update(beta,4,0,8);
   assign_u2ud();
   read_cnfg(name);

   ranlxs(rs2,100);
   ranlxd(rd2,100);

   dmax=0.0;

   for (i=0;i<100;i++)
   {
      d=fabs((double)(rs1[i]-rs2[i]));
      if (d>dmax)
         dmax=d;

      d=fabs(rd1[i]-rd2[i]);
      if (d>dmax)
         dmax=d;
   }

   ru=(double*)(u);
   rv=(double*)(v);

   for (i=0;i<(72*VOLUME);i++)
   {
      d=fabs((*ru)-(*rv));
      if (d>dmax)
         dmax=d;
      ru+=1;
      rv+=1;
   }

   error_chk();
   MPI_Reduce(&dmax,&d,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);

   if (my_rank==0)
   {
      printf("Maximal deviation = %.1e\n",d);
      printf("(should be exactly equal to 0.0)\n\n");
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
