
/*******************************************************************************
*
* File check4.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Importing a configuration previously exported by check3, possibly with
* periodic extension
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "misc.h"
#include "start.h"
#include "update.h"
#include "global.h"


int main(int argc,char *argv[])
{
   int my_rank,l[4];
   float beta;
   double plaq0,plaq1,plaq2;
   char name[80];
   FILE *log=NULL,*fin=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check4.log","w",stdout);

      printf("\n");
      printf("Importing a configuration previously exported by check3\n");
      printf("-------------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n",L0,L1,L2,L3);
      fflush(log);
   }

   beta=5.7f;
   start_ranlux(0,123456);
   geometry();
   alloc_u();
   alloc_ud();
   random_u();

   update(beta,8,0,8);
   assign_u2ud();
   plaq0=plaq_sum_dble()/(double)(6*NPROC*VOLUME);
   sprintf(name,"check3.dat");

   if (my_rank==0)
   {
      fin=fopen(name,"rb");
      error_root(fin==NULL,1,"main [check4.c]","Unable to open input file");

      fread(l,sizeof(int),4,fin);
      fread(&plaq1,sizeof(double),1,fin);

      fclose(fin);

      printf("Random configuration, average plaquette = %.12f\n\n",plaq0);
      printf("Now read configuration from file check3.dat\n");
      printf("%dx%dx%dx%d lattice\n",l[0],l[1],l[2],l[3]);
      printf("Average plaquette = %.12f\n",plaq1);
   }

   import_cnfg(name);
   plaq2=plaq_sum_dble()/(double)(6*NPROC*VOLUME);
   error_chk();

   if (my_rank==0)
   {
      printf("Check             = %.12f\n\n",plaq2);
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
