
/*******************************************************************************
*
* File time5.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of prod2su3alg and rotate_su3alg
*
*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "random.h"
#include "su3.h"
#include "start.h"
#include "misc.h"

#define N 10000000


int main(void)
{
   int count;
   double t1,t2,dt;
   su3_dble *u,*v;
   su3_alg_dble *X;
   
   printf("\n");
   printf("Timing of prod2su3alg and rotate_su3alg\n");
   printf("---------------------------------------\n\n");

   u=amalloc(2*sizeof(su3_dble),4);
   X=amalloc(sizeof(su3_alg_dble),4);   
   error((u==NULL)||(X==NULL),1,
         "main [time5.c]","Unable to allocate auxiliary arrays");

   v=u+1;
   
   rlxd_init(1,23456);
   random_su3_dble(u);
   random_su3_dble(v);
   ranlxd((double*)(X),8);
   printf("The times per application are:\n");
   
   t1=(double)clock();
   for (count=0;count<N;count++)
      prod2su3alg(u,v,X);
   t2=(double)clock();

   dt=(t2-t1)/(double)(CLOCKS_PER_SEC);
   dt*=1.0e6/(double)(N);

   printf("prod2su3alg:   %4.3f micro sec (%d Mflops)\n",dt,(int)(212.0/dt));

   t1=(double)clock();
   for (count=0;count<N;count++)
      rotate_su3alg(u,X);
   t2=(double)clock();

   dt=(t2-t1)/(double)(CLOCKS_PER_SEC);
   dt*=1.0e6/(double)(N);

   printf("rotate_su3alg: %4.3f micro sec (%d Mflops)\n\n",dt,(int)(274.0/dt));
   exit(0);
}
