
/*******************************************************************************
*
* File time3.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of inv_pauli_dble and det_pauli_dble
*
*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "random.h"
#include "sw_term.h"
#include "start.h"

#define NS 10000
#define NL 30

#if (defined SSE)
#include "sse.h"
#endif

int main(void)
{
   int n,itest;
   double t1,t2,dt,det;
   pauli_dble *mb,*m;
   
   printf("\n");
   printf("Timing of inv_pauli_dble and det_pauli_dble\n");
   printf("-------------------------------------------\n\n");

   mb=amalloc(NS*sizeof(pauli_dble),7);
   error(mb==NULL,1,"main [time3.c]",
         "Unable to allocate auxiliary arrays");
   
   rlxd_init(1,23456);

   for (m=mb;m<(mb+NS);m++)
   {
      ranlxd((*m).u,36);

      for (n=0;n<6;n++)
         (*m).u[n]=1.0;

      for (n=6;n<36;n++)
         (*m).u[n]=0.1*((*m).u[n]-0.5);
   }

   itest=0;

   t1=(double)clock();
   for (n=0;n<NL;n++)
   {
      for (m=mb;m<(mb+NS);m++)
      {
#if (defined SSE)
         _prefetch_pauli_dble(m+1);
#endif
         itest+=inv_pauli_dble(m,m);
      }
   }
   t2=(double)clock();

   dt=(t2-t1)/(double)(CLOCKS_PER_SEC);
   dt*=2.0e6/(double)(NL*NS);

   printf("Average time per lattice point:\n");
   printf("inv_pauli_dble: %4.3f micro sec ",dt);
   printf("(number of failures: %d)\n",itest/NL);

   t1=(double)clock();
   for (n=0;n<(3*NL);n++)
   {
      for (m=mb;m<(mb+NS);m++)
      {
#if (defined SSE)
         _prefetch_pauli_dble(m+1);
#endif         
         det=det_pauli_dble(m);
      }
   }
   t2=(double)clock();

   dt=(t2-t1)/(double)(CLOCKS_PER_SEC);
   dt*=2.0e6/(double)(3*NL*NS);

   printf("det_pauli_dble: %4.3f micro sec\n\n",dt);
   exit(0);
}
