
/*******************************************************************************
*
* File check2.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Direct test of the Schwarz alternating procedure
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "linalg.h"
#include "block.h"
#include "sw_term.h"
#include "dirac.h"
#include "sap_gcr.h"
#include "global.h"


int main(int argc,char *argv[])
{
   int my_rank,n;
   int bs[4],nmr,isx,isr,isv;
   float res,del;
   complex z;
   spinor *psx,*psb,*psr,*psv;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check2.log","w",stdout);
      in=freopen("check2.in","r",stdin);

      printf("\n");
      printf("Direct test of the Schwarz alternating procedure\n");
      printf("------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bs %d %d %d %d\n",&bs[0],&bs[1],&bs[2],&bs[3]);
      scanf("nmr %d",&nmr);
      fclose(in);

      printf("bs = %d %d %d %d\n",bs[0],bs[1],bs[2],bs[3]);
      printf("nmr = %d\n\n",nmr);
      fflush(log);
   }

   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmr,1,MPI_INT,0,MPI_COMM_WORLD);

   start_ranlux(0,12345);
   geometry();
   alloc_u();
   alloc_ud();
   alloc_sw();
   alloc_swd();
   alloc_s(4);

   set_gcr_parms(bs,nmr,1,1);
   set_lat_parms(0.0,0.1,0.2);
   random_ud();
   sw_term();
   assign_ud2u();
   assign_swd2sw();
   assign_ud2ubgr(GCR_BLOCKS);
   n=assign_swd2swbgr(GCR_BLOCKS,ODD_PTS);
   error(n!=0,1,"main [check2.c]","Inversion of the SW term was not safe");

   psx=ps[0][0];
   psb=ps[1][0];
   psr=ps[2][0];
   psv=ps[3][0];

   isx=0;
   isr=2;
   isv=3;

   random_s(VOLUME,psb,1.0f);
   normalize(VOLUME,1,psb);
   set_s2zero(VOLUME,psx);
   assign_s2s(VOLUME,psb,psr);
   z.im=0.0f;

   for (n=0;n<8;n++)
   {
      msap(nmr,isr,isx);
      res=norm_square(VOLUME,1,psr);
      res=(float)(sqrt((double)(res)));

      if (my_rank==0)
         printf("n = %d: \t residue = %.2e\t ",n+1,res);

      Qnohat(isx,isv);
      mulg5(VOLUME,psv);
      z.re=-1.0f;
      mulc_spinor_add(VOLUME,psv,psb,z);
      z.re=1.0f;
      mulc_spinor_add(VOLUME,psv,psr,z);
      del=norm_square(VOLUME,1,psv);
      del=(float)(sqrt((double)(del)));

      if (my_rank==0)
         printf("check = %.2e\n",del);
   }

   error_chk();

   if (my_rank==0)
   {
      printf("\n");
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
