
/*******************************************************************************
*
* File time1.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of the block inversion program blk_mres
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "update.h"
#include "linalg.h"
#include "block.h"
#include "sw_term.h"
#include "dirac.h"
#include "sap_gcr.h"
#include "global.h"


int main(int argc,char *argv[])
{
   int my_rank,n,count;
   int bs[4],nb,isw,nmr;
   double wt1,wt2,wdt;
   block_t *b0,*b,*bm;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("time1.log","w",stdout);
      in=freopen("time1.in","r",stdin);

      printf("\n");
      printf("Timing of the block inversion program blk_mres\n");
      printf("----------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bs %d %d %d %d\n",&bs[0],&bs[1],&bs[2],&bs[3]);
      scanf("nmr %d\n",&nmr);
      fclose(in);

      printf("bs = %d %d %d %d\n",bs[0],bs[1],bs[2],bs[3]);
      printf("nmr = %d\n\n",nmr);
   }

   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&nmr,1,MPI_INT,0,MPI_COMM_WORLD);

   start_ranlux(0,12345);
   geometry();
   alloc_ud();
   alloc_swd();
   alloc_s(1);
   alloc_bgr(GCR_BLOCKS,bs);

   set_lat_parms(0.0,0.1,0.2);
   random_ud();
   sw_term();
   assign_ud2ubgr(GCR_BLOCKS);
   n=assign_swd2swbgr(GCR_BLOCKS,ODD_PTS);
   error(n!=0,1,"main [time1.c]","Inversion of the SW term was not safe");

   random_s(VOLUME,ps[0][0],1.0f);
   b0=blk_list(GCR_BLOCKS,&nb,&isw);
   bm=b0+nb;

   n=2000000/(nmr*VOLUME);
   if (n<1)
      n=1;

   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();
   for (count=0;count<n;count++)
   {
      for (b=b0;b<bm;b++)
      {
         assign_s2sb(b,EVEN_PTS,0,1);
         blk_mres(b,nmr);
      }
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   if (my_rank==0)
   {
      printf("Field copying + block inversion:\n");
      wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);
      printf("Time per lattice point: %.2f micro sec",wdt);
      printf(" (about %d Mflops [%d bit arithmetic])\n",
             (int)((double)(nmr*2064)/wdt),(int)(sizeof(spinor))/3);
      printf("Time per lattice point and MR iteration: %.2f micro sec\n\n",
             wdt/(double)(nmr));
   }

   n=20000000/VOLUME;
   if (n<1)
      n=1;

   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();
   for (count=0;count<n;count++)
   {
      for (b=b0;b<bm;b++)
         assign_s2sb(b,EVEN_PTS,0,1);
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   error_chk();

   if (my_rank==0)
   {
      printf("Field copying only:\n");
      wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);
      printf("Time per lattice point: %.3f micro sec\n\n",wdt);
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
