
/*******************************************************************************
*
* File check6.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Direct check of flipbc and flipbcd
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "flags.h"
#include "random.h"
#include "start.h"
#include "update.h"
#include "global.h"

static su3 *usv;
static su3_dble *udsv;


static void save_u(void)
{
   su3 *u,*um,*v;

   u=pu[VOLUME/2][0];
   um=u+4*VOLUME;
   v=usv;

   for (;u<um;u++)
   {
      *v=*u;
      v+=1;
   }
}


static void save_ud(void)
{
   su3_dble *ud,*udm,*vd;

   ud=pud[VOLUME/2][0];
   udm=ud+4*VOLUME;
   vd=udsv;

   for (;ud<udm;ud++)
   {
      *vd=*ud;
      vd+=1;
   }
}


static int cmp_u(float c,su3 *u,su3 *v)
{
   float *ru,*rv,*rm;

   ru=(float*)(u);
   rv=(float*)(v);
   rm=ru+18;

   for (;ru<rm;ru++)
   {
      if (((*ru)-c*(*rv))!=0.0f)
         return 1;
      rv+=1;
   }

   return 0;
}


static int cmp_ud(double c,su3_dble *u,su3_dble *v)
{
   double *ru,*rv,*rm;

   ru=(double*)(u);
   rv=(double*)(v);
   rm=ru+18;

   for (;ru<rm;ru++)
   {
      if (((*ru)-c*(*rv))!=0.0)
         return 1;
      rv+=1;
   }

   return 0;
}


int main(int argc,char *argv[])
{
   int my_rank,n;
   int x0,x1,x2,x3,x[4];
   int ip,ix,mu,it,itd,it_all,itd_all;
   double a0,a1,b0,b1;
   su3 *u,*v;
   su3_dble *ud,*vd;
   FILE *log=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check6.log","w",stdout);
      printf("\n");
      printf("Direct check of flipbc and flipbcd\n");
      printf("----------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);
   }

   start_ranlux(0,12345);
   geometry();
   alloc_u();
   alloc_ud();

   random_u();
   random_ud();
   a0=plaq_sum();
   b0=plaq_sum_dble();

   if (my_rank==0)
   {
      printf("Action[ u] = % .6e\n",a0/(double)(6*NPROC*VOLUME));
      printf("Action[ud] = % .6e\n\n",b0/(double)(6*NPROC*VOLUME));
   }

   usv=amalloc(4*VOLUME*sizeof(su3),ALIGN);
   udsv=amalloc(4*VOLUME*sizeof(su3_dble),ALIGN);
   error((usv==NULL)||(udsv==NULL),1,"main [check6.c]",
         "Unable to allocate auxiliary arrays");

   for (n=0;n<2;n++)
   {
      save_u();
      save_ud();

      flipbc();
      flipbcd();

      it=0;
      itd=0;

      for (x0=0;x0<(L0*NPROC0);x0++)
      {
         for (x1=0;x1<(L1*NPROC1);x1++)
         {
            for (x2=0;x2<(L2*NPROC2);x2++)
            {
               for (x3=0;x3<(L3*NPROC3);x3++)
               {
                  x[0]=x0;
                  x[1]=x1;
                  x[2]=x2;
                  x[3]=x3;

                  ipt_global(x,&ip,&ix);

                  if ((my_rank==ip)&&(ix>=(VOLUME/2)))
                  {
                     u=pu[ix][0];
                     v=usv+8*(ix-(VOLUME/2));

                     ud=pud[ix][0];
                     vd=udsv+8*(ix-(VOLUME/2));

                     for (mu=0;mu<4;mu++)
                     {
                        if ((mu==0)&&(x0==(L0*NPROC0-1)))
                        {
                           it+=cmp_u(-1.0f,u,v);
                           itd+=cmp_ud(-1.0,ud,vd);
                        }
                        else
                        {
                           it+=cmp_u(1.0f,u,v);
                           itd+=cmp_ud(1.0,ud,vd);
                        }

                        u+=1;
                        v+=1;
                        ud+=1;
                        vd+=1;

                        if ((mu==0)&&(x0==0))
                        {
                           it+=cmp_u(-1.0f,u,v);
                           itd+=cmp_ud(-1.0,ud,vd);
                        }
                        else
                        {
                           it+=cmp_u(1.0f,u,v);
                           itd+=cmp_ud(1.0,ud,vd);
                        }

                        u+=1;
                        v+=1;
                        ud+=1;
                        vd+=1;
                     }
                  }
               }
            }
         }
      }

      MPI_Reduce(&it,&it_all,1,MPI_INT,MPI_SUM,0,MPI_COMM_WORLD);
      MPI_Reduce(&itd,&itd_all,1,MPI_INT,MPI_SUM,0,MPI_COMM_WORLD);
      MPI_Bcast(&it_all,1,MPI_INT,0,MPI_COMM_WORLD);
      MPI_Bcast(&itd_all,1,MPI_INT,0,MPI_COMM_WORLD);

      a1=fabs(1.0-plaq_sum()/a0);
      b1=fabs(1.0-plaq_sum_dble()/b0);

      if (my_rank==0)
      {
         print_flags();

         if (it_all==0)
            printf("No mismatch found (flipbc)\n");
         else
            printf("Single-precision links are not properly flipped\n");

         if (itd_all==0)
            printf("No mismatch found (flipbcd)\n");
         else
            printf("Double-precision links are not properly flipped\n");

         printf("Change in the plaquette action: %1.1e, %1.1e\n\n",a1,b1);
      }
   }

   error_chk();

   if (my_rank==0)
      fclose(log);

   MPI_Finalize();
   exit(0);
}
