
/*******************************************************************************
*
* File check4.c
*
* Copyright (C) 2005 Luigi Del Debbio, Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the programs swb_term() and invert_blk_swd()
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "flags.h"
#include "random.h"
#include "start.h"
#include "sw_term.h"
#include "block.h"
#include "global.h"

static const su3_dble ud0={{0.0}};


static int is_on_bnd(block_t *b,int ix)
{
   int vol,*up,*dn;

   vol=(*b).vol;

   if (ix>=vol)
      return 1;

   up=(*b).iup[ix];

   if ((up[0]>=vol)||(up[1]>=vol)||(up[2]>=vol)||(up[3]>=vol))
      return 1;

   dn=(*b).idn[ix];

   if ((dn[0]>=vol)||(dn[1]>=vol)||(dn[2]>=vol)||(dn[3]>=vol))
      return 1;

   return 0;
}


static void set_active2zero(void)
{
   int ix,iy,iz,ibx,iby,mu,nb,isw;
   block_t *b,*bm;

   b=blk_list(HMC_BLOCKS,&nb,&isw);
   bm=b+nb;

   for (;b<bm;b++)
   {
      for (ix=0;ix<(*b).vol;ix++)
      {
         ibx=is_on_bnd(b,ix);

         for (mu=0;mu<4;mu++)
         {
            iy=(*b).iup[ix][mu];
            iby=is_on_bnd(b,iy);
         
            if ((ibx==0)||(iby==0))
            {
               iz=(*b).imb[ix];
               *(pud[iz][mu])=ud0;
            }
         }
      }
   }

   set_flags(NEW_UD);
}


static double cmp_swb(void)
{
   int ix,iy,nb,isw;
   double *u,*um,*bu,d,dmax;
   block_t *b,*bm;

   dmax=0.0;   
   b=blk_list(HMC_BLOCKS,&nb,&isw);
   bm=b+nb;

   for (;b<bm;b++)
   {
      for (ix=0;ix<(*b).vol;ix++)
      {
         iy=(*b).imb[ix];
      
         bu=(*b).swb[2*ix].u;
         u=swd[2*iy].u;
         um=u+72;

         for(;u<um;u++)
         {
            d=fabs((*bu)-(*u));
            if (d>dmax)
               dmax=d;

            bu+=1;
         }
      }
   }

   return dmax;
}


static double cmp_swd(void)
{
   int ix,iy,nb,isw;
   double *u,*um,*bu,d,dmax;
   block_t *b,*bm;
   
   dmax=0.0;   
   b=blk_list(HMC_BLOCKS,&nb,&isw);
   bm=b+nb;

   for (;b<bm;b++)
   {
      for (ix=0;ix<(*b).vol;ix++)
      {
         iy=(*b).imb[ix];
      
         bu=(*b).swd[2*ix].u;
         u=swd[2*iy].u;
         um=u+72;

         for(;u<um;u++)
         {
            d=fabs((*bu)-(*u));
            if (d>dmax)
               dmax=d;

            bu+=1;
         }
      }
   }

   return dmax;
}


int main(int argc,char *argv[])
{
   int my_rank,bs[4],n,nb,isw,ifail,ifail_all;
   double d,dmax;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check4.log","w",stdout);
      in=freopen("check4.in","r",stdin);
      
      printf("\n");
      printf("Check of the programs swb_term() and invert_blk_swd()\n");
      printf("-----------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bs %d %d %d %d",&bs[0],&bs[1],&bs[2],&bs[3]);      
      fclose(in);

      printf("bs = %d %d %d %d\n",bs[0],bs[1],bs[2],bs[3]);      
      printf("Random gauge field\n\n");
   }

   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);
   
   start_ranlux(0,1234);
   geometry();
   alloc_ud();
   alloc_bgr(HMC_BLOCKS,bs);
   blk_list(HMC_BLOCKS,&nb,&isw);

   set_lat_parms(0.0,0.1,1.234);
   random_ud();
   assign_ud2udbgr(HMC_BLOCKS);

   swb_term();
   dmax=cmp_swd();
   set_active2zero();
   sw_term();
   d=cmp_swb();
   if (dmax>d)
      d=dmax;
   error_chk();   
   
   MPI_Reduce(&d,&dmax,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);

   if (my_rank==0)
   {
      printf("Calculation of the swb array:\n");
      printf("Maximal absolute deviation = %.2e\n\n",dmax);
   }   

   random_ud();
   sw_term();
   assign_swd2swdbgr(HMC_BLOCKS,NO_PTS);
   ifail=0;
   
   for (n=0;n<nb;n++)
      ifail|=invert_blk_swd(HMC_BLOCKS,n,EVEN_PTS);

   invert_swd(EVEN_PTS);
   d=cmp_swd();   
   MPI_Reduce(&d,&dmax,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
   MPI_Reduce(&ifail,&ifail_all,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
   error_chk();
   
   if (my_rank==0)
   {
      printf("Inversion on the even points:\n");
      printf("Maximal absolute deviation = %.2e (ifail = %d)\n",dmax,ifail_all);
   }  

   ifail=0;
   
   for (n=0;n<nb;n++)
      ifail|=invert_blk_swd(HMC_BLOCKS,n,ODD_PTS);

   invert_swd(ODD_PTS);
   d=cmp_swd();
   MPI_Reduce(&d,&dmax,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
   MPI_Reduce(&ifail,&ifail_all,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
   error_chk();   

   if (my_rank==0)
   {
      printf("Inversion on the odd points:\n");
      printf("Maximal absolute deviation = %.2e (ifail = %d)\n",dmax,ifail_all);
   }   

   random_ud();
   sw_term();
   assign_swd2swdbgr(HMC_BLOCKS,NO_PTS);
   ifail=0;
   
   for (n=0;n<nb;n++)
      ifail|=invert_blk_swd(HMC_BLOCKS,n,ALL_PTS);

   invert_swd(ALL_PTS);
   d=cmp_swd();   
   MPI_Reduce(&d,&dmax,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
   MPI_Reduce(&ifail,&ifail_all,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);   
   error_chk();
   
   if (my_rank==0)
   {
      printf("Inversion on the all points:\n");
      printf("Maximal absolute deviation = %.2e (ifail = %d)\n\n",
             dmax,ifail_all);
      fclose(log);
   }  
   
   MPI_Finalize();
   exit(0);
}
