
/*******************************************************************************
*
* File time1.c
*
* Copyright (C) 2005 Luigi del Debbio, Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of the program sw_term()
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "sw_term.h"
#include "global.h"

int main(int argc,char *argv[])
{
   int my_rank,count,n;
   double wt1,wt2,wdt;
   FILE *log=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("time1.log","w",stdout);
      printf("\n");
      printf("Timing of the program sw_term()\n");
      printf("-------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);
   }

   start_ranlux(0,12345);
   geometry();
   alloc_ud();
   alloc_swd();

   set_lat_parms(0.0,0.1,1.234);
   random_ud();
   sw_term();
   
   n=500000/VOLUME;
   if (n<1)
      n=1;

   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();
   for (count=0;count<n;count++)
      sw_term();
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   error_chk();
   wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);

   if (my_rank==0)
   {
      printf("Time per lattice point: %4.3f micro sec",wdt);
      printf(" (%d Mflops [64 bit arithmetic])\n\n",(int)(9936.0/wdt));
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
