
/*******************************************************************************
*
* File time2.c
*
* Copyright (C) 2005 Luigi Del Debbio, Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of the program blk_sw_term()
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "flags.h"
#include "random.h"
#include "start.h"
#include "sw_term.h"
#include "block.h"
#include "global.h"


int main(int argc,char *argv[])
{
   int my_rank,bs[4],n,count;
   int ib,nb,isw;
   double wt1,wt2,wdt;
   FILE *log=NULL,*in=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("time2.log","w",stdout);
      in=freopen("time2.in","r",stdin);
      
      printf("\n");
      printf("Timing of the program blk_sw_term()\n");
      printf("-----------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      scanf("bs %d %d %d %d",&bs[0],&bs[1],&bs[2],&bs[3]);      
      fclose(in);

      printf("bs = %d %d %d %d\n",bs[0],bs[1],bs[2],bs[3]);      
      printf("Random gauge field\n\n");
   }

   MPI_Bcast(bs,4,MPI_INT,0,MPI_COMM_WORLD);
   
   start_ranlux(0,1234);
   geometry();
   alloc_ud();
   alloc_bgr(HMC_BLOCKS,bs);
   blk_list(HMC_BLOCKS,&nb,&isw);

   set_lat_parms(0.0,0.1,1.234);
   random_ud();
   assign_ud2udbgr(HMC_BLOCKS);
   swb_term();

   n=500000/VOLUME;
   if (n<1)
      n=1;

   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();
   for (count=0;count<n;count++)
   {
      for (ib=0;ib<nb;ib++)
         blk_sw_term(ib);
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   error_chk();
   wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);

   if (my_rank==0)
   {
      printf("Time per lattice point: %4.3f micro sec\n\n",wdt);
      fclose(log);
   }
   
   MPI_Finalize();
   exit(0);
}
