
/*******************************************************************************
*
* File check10.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the program for the plaquette action of the single-precision
* gauge field
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "flags.h"
#include "misc.h"
#include "update.h"
#include "global.h"

static int nfc[8],ofs[8];
static su3 *g,*gbuf;


static void pack_gbuf(void)
{
   int n,ix,iy,io;

   nfc[0]=FACE0/2;
   nfc[1]=FACE0/2;
   nfc[2]=FACE1/2;
   nfc[3]=FACE1/2;
   nfc[4]=FACE2/2;
   nfc[5]=FACE2/2;
   nfc[6]=FACE3/2;
   nfc[7]=FACE3/2;

   ofs[0]=0;
   ofs[1]=ofs[0]+nfc[0];
   ofs[2]=ofs[1]+nfc[1];
   ofs[3]=ofs[2]+nfc[2];
   ofs[4]=ofs[3]+nfc[3];
   ofs[5]=ofs[4]+nfc[4];
   ofs[6]=ofs[5]+nfc[5];
   ofs[7]=ofs[6]+nfc[6];

   for (n=0;n<8;n++)
   {
      io=ofs[n];

      for (ix=0;ix<nfc[n];ix++)
      {
         iy=map[io+ix];
         gbuf[io+ix]=g[iy];
      }
   }
}


static void send_gbuf(void)
{
   int n,mu,np,saddr,raddr;
   int nbf,tag;
   float *sbuf,*rbuf;
   MPI_Status stat;

   for (n=0;n<8;n++)
   {
      nbf=18*nfc[n];

      if (nbf>0)
      {
         tag=mpi_tag();
         mu=n/2;
         np=cpr[mu];

         if (n==(2*mu))
         {
            saddr=npr[n+1];
            raddr=npr[n];
         }
         else
         {
            saddr=npr[n-1];
            raddr=npr[n];
         }

         sbuf=(float*)(gbuf+ofs[n]);
         rbuf=(float*)(g+ofs[n]+VOLUME);

         if ((np|0x1)!=np)
         {
            MPI_Send(sbuf,nbf,MPI_FLOAT,saddr,tag,MPI_COMM_WORLD);
            MPI_Recv(rbuf,nbf,MPI_FLOAT,raddr,tag,MPI_COMM_WORLD,&stat);
         }
         else
         {
            MPI_Recv(rbuf,nbf,MPI_FLOAT,raddr,tag,MPI_COMM_WORLD,&stat);
            MPI_Send(sbuf,nbf,MPI_FLOAT,saddr,tag,MPI_COMM_WORLD);
         }
      }
   }
}


static void random_g(void)
{
   int ix;

   for (ix=0;ix<VOLUME;ix++)
      random_su3(g+ix);

   pack_gbuf();
   send_gbuf();
}


static void transform_u(void)
{
   int ix,iy,mu;
   su3 *ub,u,v,w,gx,gxi,gy,gyi;

   for (ix=(VOLUME/2);ix<VOLUME;ix++)
   {
      ub=pu[ix][0];
      gx=g[ix];

      for (mu=0;mu<4;mu++)
      {
         iy=iup[ix][mu];
         gy=g[iy];
         u=ub[2*mu];
         _su3_dagger(gyi,gy);
         _su3_times_su3(v,u,gyi);
         _su3_times_su3(w,gx,v);
         ub[2*mu]=w;

         iy=idn[ix][mu];
         gy=g[iy];
         u=ub[2*mu+1];
         _su3_dagger(gxi,gx);
         _su3_times_su3(v,u,gxi);
         _su3_times_su3(w,gy,v);
         ub[2*mu+1]=w;
      }
   }

   set_flags(NEW_U);
}


static void random_vec(int *svec)
{
   int mu,bs[4];
   double r[4];

   bs[0]=NPROC0*L0;
   bs[1]=NPROC1*L1;
   bs[2]=NPROC2*L2;
   bs[3]=NPROC3*L3;

   ranlxd(r,4);

   for (mu=0;mu<4;mu++)
   {
      svec[mu]=(int)((double)(bs[mu])*r[mu]);
      if (svec[mu]>(bs[mu]/2))
         svec[mu]-=bs[mu];
   }
   
   MPI_Bcast(svec,4,MPI_INT,0,MPI_COMM_WORLD);   
}


int main(int argc,char *argv[])
{
   int my_rank;
   int n,s[4];
   double p1,p2;
   FILE *log=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check10.log","w",stdout);
      printf("\n");
      printf("Plaquette action of the single-precision gauge field\n");
      printf("----------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);
   }

   start_ranlux(0,12345);
   geometry();
   alloc_u();
   p1=plaq_sum();

   if (my_rank==0)
   {
      printf("Average plaquette after initialization = %.8e\n",
             p1/(double)(6*VOLUME*NPROC));
      printf("(should be equal to 3.0)\n\n");
   }

   print_flags();
   g=amalloc(NSPIN*sizeof(su3),4);
   if (BNDRY!=0)
      gbuf=amalloc((BNDRY/2)*sizeof(su3),4);

   error((g==NULL)||((BNDRY!=0)&&(gbuf==NULL)),1,"main [check10.c]",
         "Unable to allocate auxiliary arrays");

   random_u();
   p1=plaq_sum();
   random_g();
   transform_u();
   p2=plaq_sum();

   if (my_rank==0)
   {
      printf("Gauge invariance: relative difference = %.1e\n",
             fabs(1.0-p1/p2));
      printf("(should be at most 10^(-6) or so)\n\n");
   }

   p1=plaq_sum();
   free_ucom_bufs(1,0);
   p2=plaq_sum();

   if (my_rank==0)
   {
      printf("Free & reallocate buffers: relative difference = %.1e\n",
             fabs(1.0-p1/p2));
      printf("(should be 0.0)\n\n");

      printf("Translation invariance:\n");
   }

   for (n=0;n<8;n++)
   {
      random_vec(s);
      shift_u(s);
      p2=plaq_sum();

      if (my_rank==0)
      {
         printf("s=(% d,% d,% d,% d), ",s[0],s[1],s[2],s[3]);
         printf("relative deviation = %.1e\n",fabs(1.0-p1/p2));
      }
   }

   for (n=0;n<5;n++)
      update(5.7f,0,1,6);

   p2=plaq_sum();

   if (my_rank==0)
   {
      printf("\n");
      printf("Relative change after 5 over-relaxation sweeps = %.1e\n\n",
             fabs(1.0-p1/p2));
      fclose(log);
   }

   MPI_Finalize();
   exit(0);
}
