
/*******************************************************************************
*
* File check7.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check that all link variables are treated approximately equally by the
* update program
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "update.h"
#include "global.h"


static void shift_field(void)
{
   int np[4],s[4],mu;
   double r[4];

   np[0]=NPROC0;
   np[1]=NPROC1;
   np[2]=NPROC2;
   np[3]=NPROC3;

   ranlxd(r,4);

   for (mu=0;mu<4;mu++)
   {
      if (np[mu]>1)
         s[mu]=(int)(2.0*r[mu])+1;
      else
         s[mu]=0;
   }
   
   MPI_Bcast(s,4,MPI_INT,0,MPI_COMM_WORLD);

   shift_u(s);
}


int main(int argc,char *argv[])
{
   int my_rank,ix,mu,n,nlks,k,nms;
   int ifc,mn[4],mx[4];
   double avg[4],dev[4],r;
   su3 *u,*um;
   link_t *lks;
   FILE *log=NULL;   

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      log=freopen("check7.log","w",stdout);
   
      printf("\n");
      printf("Average number of times the links are visited\n");
      printf("---------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);      
   }   
   
   start_ranlux(0,12345);
   geometry();
   alloc_u();
   lks=active_links(&nlks);
   nms=100;
   
   if (my_rank==0)
   {
      printf("Number of sweeps: %d\n",nms);
      printf("Number of active links: %d\n",nlks);
      printf("Total number of links:  %d\n",4*VOLUME);
   }
   
   for (k=0;k<nms;k++)
   {
      for (n=0;n<nlks;n++)
      {
         ix=lks[n].ix;
         mu=lks[n].mu;
         u=pu[ix][mu];
         (*u).c12.re+=1.0f;
      }

      shift_field();
   }

   free_shift_bufs(1,1);
   error_chk();

   if (my_rank==0)
   {
      for (mu=0;mu<4;mu++)
      {
         avg[mu]=0.0;
         dev[mu]=0.0;
         mn[mu]=nms;
         mx[mu]=0;
      }

      u=pu[VOLUME/2][0];
      um=u+4*VOLUME;
   
      for (;u<um;)
      {
         for (ifc=0;ifc<8;ifc++)
         {
            mu=ifc/2;
            r=(double)((*u).c12.re);
            n=(int)(r);
            
            avg[mu]+=r;
            dev[mu]+=r*r;
            
            if (n<mn[mu])
               mn[mu]=n;
            if (n>mx[mu])
               mx[mu]=n;

            u+=1;
         }
      }

      printf("\n");
      printf("Minimum/Average/Maximum visiting frequency:\n");

      for (mu=0;mu<4;mu++)
      {
         avg[mu]/=(double)(VOLUME);
         dev[mu]/=(double)(VOLUME);
         dev[mu]-=(avg[mu]*avg[mu]);
         
         printf("mu=%d,  %d/%.1f/%d, ",mu,mn[mu],avg[mu],mx[mu]);
         printf("standard deviation = %.1f\n",sqrt(dev[mu]));
      }

      printf("\n");
      fclose(log);
   }
   
   MPI_Finalize();    
   exit(0);
}
