
/*******************************************************************************
*
* File time1.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of the staples program
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "update.h"
#include "global.h"

#if ((defined SSE)||(defined SSE2))
static su3 r[4*VOLUME] __attribute__ ((aligned (16)));
#else
static su3 r[4*VOLUME];
#endif


int main(int argc,char *argv[])
{
   int my_rank,n,count,nlks;
   double wt1,wt2,wdt;
   link_t *lks,*lk,*lm;
   su3 *u;
   FILE *log=NULL;   

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
   
   if (my_rank==0)
   {
      log=freopen("time1.log","w",stdout);
 
      printf("\n");
      printf("Timing of the computation of the 'staples'\n");
      printf("------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);      
   }

   start_ranlux(0,12345);
   geometry();
   alloc_u();
   lks=active_links(&nlks);
   random_u();

   n=2000000/nlks;
   if (n<2)
      n=2;
   
   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();     
   for (count=0;count<n;count++)
   {
      u=r;
      lk=lks;
      lm=lks+nlks;
      
      for (;lk<lm;lk++)
      {
         staples((*lk).ix,(*lk).mu,u);
         u+=1;
      }
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();

   wdt=1.0e6*(wt2-wt1)/(double)(n*nlks);

   if (my_rank==0)
   {
      printf("Time per link: %4.3f micro sec",wdt);
      printf(" (%d Mflops [%d bit arithmetic])\n\n",
             (int)(1926.0f/wdt),((int)sizeof(su3)*4)/9);   
      fclose(log);
   }   

   MPI_Finalize();   
   exit(0);
}
