
/*******************************************************************************
*
* File time5.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Timing of the double-precision shift and action programs
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "su3.h"
#include "random.h"
#include "start.h"
#include "update.h"
#include "global.h"


static void random_vec(int *svec)
{
   int mu,bs[4];
   double r[4];

   bs[0]=NPROC0*L0;
   bs[1]=NPROC1*L1;
   bs[2]=NPROC2*L2;
   bs[3]=NPROC3*L3;

   ranlxd(r,4);

   for (mu=0;mu<4;mu++)
      svec[mu]=(int)((double)(bs[mu])*r[mu]);
   
   MPI_Bcast(svec,4,MPI_INT,0,MPI_COMM_WORLD);   
}


int main(int argc,char *argv[])
{
   int my_rank,n,count;
   int ns,svec[4];
   double pls,wt1,wt2,wdt;
   FILE *log=NULL;   

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
   
   if (my_rank==0)
   {
      log=freopen("time5.log","w",stdout);

      printf("\n");
      printf("Timing of the double-precision shift and action programs\n");
      printf("--------------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);
   }

   start_ranlux(0,12345);
   geometry();
   alloc_ud();
   random_ud();
   
   n=50000/VOLUME;
   if (n<2)
      n=2;   

   pls=plaq_sum_dble();
   
   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();    
   for (count=0;count<n;count++)
      pls=plaq_sum_dble();
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();  

   wdt=1.0e6*(wt2-wt1)/(double)(n*VOLUME);

   if (my_rank==0)
   {
      printf("Calculation of the plaquette action:\n");
      printf("Time per lattice point: %4.3f micro sec\n\n",wdt);
   }

   n=1000000/VOLUME;
   n/=(NPROC0*L0+NPROC1*L1+NPROC2*L2+NPROC3*L3);
   if (n<2)
      n=2;

   ns=0;
   
   MPI_Barrier(MPI_COMM_WORLD);
   wt1=MPI_Wtime();
   for (count=0;count<n;count++)
   {
      random_vec(svec);
      ns+=shift_ud(svec);
   }
   MPI_Barrier(MPI_COMM_WORLD);
   wt2=MPI_Wtime();  

   wdt=1.0e6*(wt2-wt1)/(double)(VOLUME*ns);

   if (my_rank==0)
   {
      printf("Translation of the gauge field:\n");
      printf("Average time per unit shift and lattice point: ");
      printf("%4.3f micro sec\n\n",wdt);
      fclose(log);
   }   

   MPI_Finalize();
   exit(0);
}
