
/*******************************************************************************
*
* File hmc.h
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
*******************************************************************************/

#ifndef HMC_H
#define HMC_H

#ifndef SU3_H
#include "su3.h"
#endif

#ifndef BLOCK_H
#include "block.h"
#endif

typedef struct
{
   int nlks,nplq,nbnd;
   int *lks,*plq,*pts,*imb,*ifc;
   su3_alg_dble *mom,*frc;
   spinor_dble *phi,*chi;
} hmc_t;

#ifndef BNDOP_C
extern void random_chi();
extern void assign_chi2sd(int k);
extern void assign_sd2chi(int k);
extern void sub_assign_PdOs(int k,int l);
extern void ROp(int *status,int *ifail);
#endif

#ifndef BSHIFT_C
extern int start_bshift(int *bs);
extern void next_bshift(int *svec);
#endif

#ifndef FORCE0_C
extern void hmc_frg(int n);
extern double hmc_acg(int n);
#endif

#ifndef FORCE1_C
extern void hmc_frf(int n,int *status,int *ifail);
extern double hmc_acf(int n,int iphi,int idet,int *status,int *ifail);
#endif

#ifndef FORCE2_C
extern void hmc_gcr_vec(int k1,int k2,int *status,int *ifail);
extern void hmc_gcr_frf(int n,int k1,int k2);
extern double hmc_gcr_acf(int k1,int *status,int *ifail);
#endif

#ifndef FRCFCTS_C
extern void det2u3alg(pauli_dble *m,u3_alg_dble *X);
extern void vec2u3alg(spinor_dble *r,spinor_dble *s,u3_alg_dble *X);
extern void (*spin2mat[])(spinor_dble *rx,spinor_dble *ry,
                          spinor_dble *sx,spinor_dble *sy,su3_dble *u);
#endif

#ifndef GENFRC_C
extern int sw_frc(int n,int k,int l,int idet);
extern void hop_frc(int n,int k,int l);
#endif

#ifndef HMCFLDS_C
extern void alloc_hmcflds(void);
extern void free_hmcflds(void);
extern hmc_t *hmcflds(void);
#endif

#ifndef LIEALG_C
extern void ExpX(double t,su3_alg_dble *X,su3_dble *u);
extern void random_alg(int n,su3_alg_dble *X);
extern double norm_square_alg(int n,su3_alg_dble *X);
extern void set_alg2zero(int n,su3_alg_dble *X);
#endif

#ifndef LPFROG_C
extern double start_hmd(int *stat_rop,int *ifail);
extern double run_hmd(int *stat_acf,int *stat_gcr_acf,
                      int *stat_frf,int *stat_gcr_frf,int *ifail);
extern int accept_step(double act0,double act1);
#endif

#ifndef MVFLD_C
extern int mvfld(void);
#endif

#endif
