
/*******************************************************************************
*
* File mutils.c
*
* Copyright (C) 2005 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Utility functions used in main programs
*
* The externally accessible functions are
*
*   int find_opt(int argc,char *argv[],char *opt)
*     Returns the position of the option opt in the array argv[]. Only
*     the elements argv[1],..,argv[argc-1] are scanned and 0 is returned
*     if opt is not found
*
*   void check_dir(char* dir)
*     Checks whether the directory dir can be accessed from all processes,
*     and if not aborts the main program with an informative error message
*
*   void check_dir_root(char* dir)
*     Checks whether the directory dir can be accessed from process 0, and
*     if not aborts the main program with an informative error message
*
*******************************************************************************/

#define MUTILS_C

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include "mpi.h"
#include "flags.h"
#include "start.h"
#include "global.h"


int find_opt(int argc,char *argv[],char *opt)
{
   int k;

   for (k=1;k<argc;k++)
      if (strcmp(argv[k],opt)==0)
         return k;

   return 0;
}


void check_dir(char* dir)
{
   int my_rank,n;
   char *tmp_file;
   FILE *tmp;

   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   n=strlen(dir);
   tmp_file=malloc(n+7+2*sizeof(int));
   error(tmp_file==NULL,1,"check_dir [mutils.c]",
         "Unable to allocate auxiliary space");
   sprintf(tmp_file,"%s/.tmp_%x",dir,my_rank);

   n=0;
   tmp=fopen(tmp_file,"rb");

   if (tmp==NULL)
   {
      n=1;
      tmp=fopen(tmp_file,"wb");
   }

   error_loc(tmp==NULL,1,"check_dir [mutils.c]",
         "Unable to access directory %s",dir);
   error_chk();

   fclose(tmp);
   if (n==1)
      remove(tmp_file);
   free(tmp_file);
}


void check_dir_root(char* dir)
{
   int my_rank,n;
   char *tmp_file;
   FILE *tmp;

   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      n=strlen(dir);
      tmp_file=malloc((n+6)*sizeof(char));
      error_root(tmp_file==NULL,1,"check_dir_root [mutils.c]",
                 "Unable to allocate auxiliary space");
      sprintf(tmp_file,"%s/.tmp",dir);

      n=0;
      tmp=fopen(tmp_file,"rb");

      if (tmp==NULL)
      {
         n=1;
         tmp=fopen(tmp_file,"wb");
      }

      error_root(tmp==NULL,1,"check_dir_root [mutils.c]",
                "Unable to access directory %s from process 0",dir);

      fclose(tmp);
      if (n==1)
         remove(tmp_file);
      free(tmp_file);
   }
}
